#uses "fwXML/fwXML.ctl"
#uses "fwTrending/fwTrending.ctl"
#uses "unImportValidation/unImportValidationDeprecated.ctl"

/**@name LIBRARY: unImportValidation.ctl

@author: Herve Milcent (AB/CO)
@co-author: Nikolaos Vamvoukas


Creation Date: 15/06/2005

Modification History:
  22/06/2018: Mariusz
    - Move deprecated functions to unImportValidationDeprecated.ctl library, whitelist import and check tag functions  
    
  08/04/2013: Herve
  - IS-1666: unImportValidation does not add default "system name" in dpe of Plot config
  - IS-1667: plot configuration, replace "#local:" by the current system name
  
  23/07/2013: Nikolaos Vamvoukas
    - New functions for the UNICOS Application Validation tool

 01/07/2013: Herve
    - IS-1155: New component: include the Validation tool into the UNICOS framework

  25/10/2010: Herve
    - IS-421: Import/Export TrendTree/WindowTree: replace if required the local system name by #local in the DPE description
    modified function: unImportValidation_FwTrendingPlot_checkTagSectionData, unImportValidation_FwTrendingPlot_importTagSectionData

  30/10/2008: Herve
    - bug wrong icons fixed
    
  19/09/2008: Herve
    - missing marker type in export
    - back compatibility with previous version
    
	05/02/2007: Herve
		- bug in unImportValidation_getTagValue: must look for </
		
	04/12/2006: Herve
		- bug if import FwTrendingPlot with no DPE
		
	29/06/2006: Herve
		- unImportValidation__FwTreeNode_importTagSectionData: if parent == force set node.parent to "", bug fwTree.ctl
		
	07/07/2005: Herve
		- exceptionInfo overwritten with evalScript, modified function: unImportValidation_importTagSectionData
	
	31/10/2006: Herve
		- add Export code in case of dsUserData is ""
		- do not replace #local in all the checkFunction for plot
		
version 1#

External Function :
        
Internal Functions:
        
Purpose:
This library contains the function used to export the tree and tree node linked to FwTrendingPlot, FwTrendingPage and _UnPanel devices

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
  . UNICOS-JCOP framework
  . PVSS version: 3.0
  . operating system: WXP.
  . distributed system: yes.
*/

// Constants
// global variable 

//@{

const int IMPORTTREE_TREE_MAX_CHAR = 10000;
const int IMPORTTREE_TREE_NODE_TYPE = 1;
const int IMPORTTREE_TREE_LINE_NUMBER = 2;
const int IMPORTTREE_TREE_FILENAME = 3;

//global mapping g_mTreeDeviceData, g_mDeviceNavigationData;
global dyn_string g_dsTreeNode, g_dsPanel, g_dsPage, g_dsPlot, g_dsNavigation;
global dyn_dyn_string g_ddsTreeNodeData, g_ddsPanelData, g_ddsPageData, g_ddsPlotData, g_ddsNavigationData;

global dyn_string g_dsTreeNode_Copy, g_dsPanel_Copy, g_dsPage_Copy, g_dsPlot_Copy, g_dsNavigation_Copy;
global dyn_dyn_string g_ddsTreeNodeData_Copy, g_ddsPanelData_Copy, g_ddsPageData_Copy, g_ddsPlotData_Copy, g_ddsNavigationData_Copy;

//------------------------------------------------------------------------------------------------------------------------

// unImportValidation_initialize
/**
Purpose:
Executes the initialization of the unTreeWidget used to view the list of trees. This function is executed at startup of the unTreeWidget.
The unTreeWidget tree is disabled during the operation. This function is started in a separate thread.

	@param sGraphTree: string, input: the name of the unTreeWidget graphical element
	@param sBar: string, input: the name of the the Microsoft ProgressBar Control, version 6.0 graphical element
	@param bClipboard: bool, input: true show the clipbpard, false don't show the clipboard

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. the JCOP hierarchy management
	. to be used with the unTreeWidget and 
	the Microsoft ProgressBar Control, version 6.0
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unImportValidation_initialize(string sGraphTree, string sBar, bool bClipboard)
{
  string imageClassParameters;
  shape aShape=getShape(sGraphTree);
  dyn_string exceptionInfo;

  unTrendTree_loadImage("", "", exceptionInfo);
  unWindowTree_loadImage("", "", exceptionInfo);
  unTreeWidget_setPropertyValue(aShape, "clearTree", true, true);
  unTreeWidget_initialize(sGraphTree, sBar, bClipboard);
}

//------------------------------------------------------------------------------------------------------------------------

// unImportValidation_EventNodeLeft
/**
Purpose:
Function called called when a left click on a node of the unTreeWidget is done during the operation of the 
tree hierarchy. This function get the operation panel of the node, show it in the main frame.

	@param sSelectedKey: string, input: the key of the node that was selected in the unTreeWidget, the format of the sSelectedKey is sContext|childrenLabel@parentKey
	@param sMode: string, input: the type of mouse click
	@param sCompositeLabel: string, input: the composite label of the node
	@param sGraphTree: string, input: the name of the unTreeWidget graphical element
	@param sBar: string, input: the name of the the Microsoft ProgressBar Control, version 6.0 graphical element
	@param bClipboard: bool, input: true show the clipboard, false don't show the clipboard
	@param iPosX: int, input: x position of the mouse
	@param iPosY: int, input: y position of the mouse

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. the JCOP hierarchy management
	. to be used with the unTreeWidget and 
	the Microsoft ProgressBar Control, version 6.0
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{EvalScript}
*/
unImportValidation_EventNodeLeft(string sSelectedKey, string sMode, string sCompositeLabel, string sGraphTree, string sBar, bool bClipboard, int iPosX, int iPosY)
{
	string sDpName, sDisplayNodeName, sParentDpName, sParentNodeName, sNodeName;
	dyn_string exInfo;
	shape aShape;
	int id;
	string sParentKey, sCurrentSelectedNodeKey, sTopDisplayName;
	int iCurrentPointer, iMousePointer;
	string sFunctionName;
		
//DebugN("unImportValidation_EventNodeLeft", sSelectedKey, sMode, sCompositeLabel, sGraphTree, sBar, bClipboard, iPosX, iPosY);
	
// start the progress bar
	id = unProgressBar_start(sBar);	
	aShape = getShape(sGraphTree);
//!!!!!!!!!!!!!! Do not disable the unTreeWidget tree because otherwise the dbl click to expand or collapse will not work.
//	unTreeWidget_setPropertyValue(aShape, "Enabled", false, true);
	iCurrentPointer = unTreeWidget_getPropertyValue(aShape, "getMousePointer", true);
	iMousePointer = unTreeWidget_ccHourglass;
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iMousePointer, true);
	
// The unTreeWidget is not disable to allow a doubleClick as also expand

	sParentKey = unTreeWidget_getPropertyValue(aShape, "selectedParent", true);
	sCurrentSelectedNodeKey = unTreeWidget_getPropertyValue(aShape, "getNodeSelectedkey", true);

	unTreeWidget_getNodeNameAndNodeDp(sCompositeLabel, sDpName, sDisplayNodeName, sParentDpName, sParentNodeName);
// get the device name & check it exists
	sNodeName = _fwTree_getNodeName(sDpName);

	switch(sMode){
		case "NodeLeftClick":
			if(dpExists("fwTT_"+sNodeName))
				dpTreeName.text = "fwTT_"+sNodeName;
			else
				dpTreeName.text = "";
			nodeName.text = sNodeName;
			dpNodeName.text = sDpName;
			break;
		default:
			break;
	}
	
// handle any errors here
	if(dynlen(exInfo)>0)
		fwExceptionHandling_display(exInfo);

// stop the progress bar
	unProgressBar_stop(id, sBar);
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iCurrentPointer, true);
//!!!!!!!!!!! as the unTreeWidget is not enabled then do not re-enabled it.
//	unTreeWidget_setPropertyValue(aShape, "Enabled", true, true);
}

//------------------------------------------------------------------------------------------------------------------------

// unImportValidation_checkTagSectionData
/**
Purpose:
Check the all the data of a tag, trigger the device type check function based on the tag name

	@param bTreeNodeDevice: bool, input: true=the function is called from a treenode and the tree node device name is given
	@param sTreeNodeDeviceName: string, input: the tree node device name
	@param sTagName: string, input: the tag name without < and >
	@param dsData: dyn_string, input: the data for the tag
	@param bKeepLocalSystem: bool, input, keep the local system name, do not replace it by #local
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_checkTagSectionData(bool bTreeNodeDevice, string sTreeNodeDeviceName, string sTagName, dyn_string dsData, 
                                  bool bKeepLocalSystem, dyn_string &exInfo, int iLineNumber, string sFileName, bool bKeepInMemory)
{
	string sFunction;
	dyn_string dsReturn;
// call the device function to export the device content
  
	if(sTagName != "") {
    switch(sTagName)
    {
      case "FwTrendingPage":
//        DebugN("FwTrendingPage..", sTagName, sTreeNodeDeviceName, dsData);
        break;
      case "FwTrendingPlot":
//        DebugN("FwTrendingPlot..", sTagName, sTreeNodeDeviceName, dsData);
        break;
      case "_FwTreeNode":
//        DebugN("_FwTreeNode..", sTagName, sTreeNodeDeviceName, dsData);
        break;
      case "_UnPanel":
//        DebugN("_UnPanel..", sTagName, sTreeNodeDeviceName, dsData);
        break;
      case "devicenavigation":
//        DebugN("devicenavigation..", sTagName, sTreeNodeDeviceName, dsData);
        break;
      default:
        DebugN("bizarre..", sTagName, dsData);
        break;
    	}
    }
		sFunction = "unImportValidation_"+sTagName+"_checkTagSectionData";
		if(isFunctionDefined(sFunction)) {
			evalScript(dsReturn, "dyn_string main(bool bTreeNodeDevice, string sTreeNodeDeviceName, dyn_string dsData, bool bKeepLocalSystem, int iLineNumber, string sFileName, bool bKeepInMemory)" + 
															"{"+
															"	dyn_string exInfo;"+
															"	"+sFunction+"(bTreeNodeDevice, sTreeNodeDeviceName, dsData, bKeepLocalSystem, exInfo, iLineNumber, sFileName, bKeepInMemory);"+
															"	return exInfo;"+
															"}",
															makeDynString(), bTreeNodeDevice, sTreeNodeDeviceName, dsData, bKeepLocalSystem, iLineNumber, sFileName, bKeepInMemory);
			dynAppend(exInfo, dsReturn);
		}
		else
			fwException_raise(exInfo, "ERROR", "unImportValidation_checkTagSectionData: unknown tag name function "+sFunction+" for tag "+sTagName, "");
}

//------------------------------------------------------------------------------------------------------------------------

// unImportValidation_importTagSectionData
/**
Purpose:
Check the all the data of a tag, trigger the device type import function based on the tag name

	@param sTagName: string, input: the tag name without < and >
	@param dsData: dyn_string, input: the data for the tag
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_importTagSectionData(string sTagName, dyn_string dsData, dyn_string &exInfo)
{
	string sFunction;
	dyn_string dsReturn;
	
// call the device function to export the device content
	if(sTagName != "") {
		sFunction = "unImportValidation_"+sTagName+"_importTagSectionData";
		if(isFunctionDefined(sFunction)) {
			evalScript(dsReturn, "dyn_string main(dyn_string dsData)" + 
															"{"+
															"	dyn_string exInfo;"+
															"	"+sFunction+"(dsData, exInfo);"+
															"	return exInfo;"+
															"}",
															makeDynString(), dsData);
			dynAppend(exInfo, dsReturn);
		}
		else
			fwException_raise(exInfo, "ERROR", "unImportValidation_importTagSectionData: unknown tag name function "+sFunction+" for tag "+sTagName, "");
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation__FwTreeNode_checkTagSectionData
/**
Purpose:
Check all the data of the _FwTreeNode

	@param bTreeNodeDevice: bool, input: true=the function is called from a treenode and the tree node device name is given
	@param sTreeNodeDeviceName: string, input: the tree node device name
	@param dsData: dyn_string, input: the data for the tag
	@param bKeepLocalSystem: bool, input, keep the local system name, do not replace it by #local
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation__FwTreeNode_checkTagSectionData(bool bTreeNodeDevice, string sTreeNodeDeviceName, dyn_string dsData, bool bKeepLocalSystem, 
                                              dyn_string &exInfo, int iLineDevice, string sFileName, bool bKeepInMemory)
{
	string sValue, sTag, sNodeName, sParentNodeName, sCurrentParentNodeName, sDeviceName, sDeviceType, sCurrentDeviceName, sCurrentDeviceType, sConf;
	dyn_string dsChildren, dsDevData, dsUserData, dsSplit, dsNavRebuilt, dsNavData;
	int i, len, iNodeCU, iCurrentNodeCU;
	int iDataIndex, iNavIndex;
  dyn_string dsDataToKeep;
  string sDeviceData;
	
//DebugN("unImportValidation__FwTreeNode_checkTagSectionData", dsData[1], iLineDevice, bKeepInMemory);	
// check the size: minimum 5
	if(dynlen(dsData) < 5) {
		fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong data length", "");
	}
	else {
// get the name and check it
		sValue = unImportValidation_getTagValue(dsData, "<name>");
		sNodeName = sValue;
//DebugN("-->*****", sValue, sNodeName, sParentNodeName);
		if(!unImportValidation_checkNodeName(sValue) || (sValue == "")) {
			fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong node name, "+sValue, "");
		}
// get the parent and check it
		sParentNodeName = unImportValidation_getTagValue(dsData, "<parentname>");
//DebugN("???? parent node name", sParentNodeName);
		if(!unImportValidation_checkNodeName(sParentNodeName)) {
			fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong parent node name, "+sParentNodeName, "");
		}
		if(sParentNodeName == sNodeName) {
			fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong parent node name = node name, "+sParentNodeName, "");
		}
// check that the tree node device name and the tree name are the same, check the parent node name is ""
		if(bTreeNodeDevice) {
			if(sTreeNodeDeviceName != sNodeName)
				fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: tree name and treenode name are different, "+sTreeNodeDeviceName+", "+sNodeName, "");
			if(sParentNodeName != "")
				fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: first node of a tree, wrong parent nodename, "+sTreeNodeDeviceName+", "+sParentNodeName, "");
		}
// get the children and check them
		sValue = unImportValidation_getTagValue(dsData, "<children>");
		dsChildren = strsplit(sValue, UN_PARAMETER_DELIMITER);
		len = dynlen(dsChildren);
		for(i=1;i<=len;i++) {
			if(!unImportValidation_checkNodeName(dsChildren[i]) || (dsChildren[i] == "")) {
				fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong children node name, "+dsChildren[i], "");
			}
		}
// get the device name and check them
		sValue = unImportValidation_getTagValue(dsData, "<devicedata>");
    sDeviceData = sValue;
		dsChildren = strsplit(sValue, UN_PARAMETER_DELIMITER);
		if(dynlen(dsChildren) < 3)
			fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong device config, "+sValue, "");
		else {
			sDeviceName = dsChildren[1];
			sDeviceType = dsChildren[2];
			sscanf(dsChildren[3], "%d", iNodeCU);
// check the device name
			if(!unImportValidation_checkDpName(dsChildren[1])) {
				fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong device dp name, "+dsChildren[1], "");
			}
		}
// check the device data
		dsChildren = makeDynString();
		iDataIndex = 1;
		unImportValidation_getTagSectionData(dsData, dsChildren, iDataIndex, "<deviceconfiguration>", "</deviceconfiguration>");
		if(dynlen(dsChildren) > 0) { // there is a device so check it
			iDataIndex = 1;
			unImportValidation_getNextTagSectionData(dsChildren, dsDevData, iDataIndex, sTag);
//DebugN("!!!!!!!!!!! unImportValidation_treenode_checkTagSectionData check device config", sTag, dsChildren, dsDevData);
			unImportValidation_checkTagSectionData(true, sDeviceName, unImportValidation_getTagName(sTag), dsDevData, bKeepLocalSystem, exInfo, iLineDevice, sFileName, bKeepInMemory);
		}
    if(bKeepInMemory)
    {
//global mapping g_mTreeDeviceData;
//global dyn_string g_dsTreeNode, g_dsPanel, g_dsPage, g_dsPlot, g_dsNavigation;
//const int IMPORTTREE_TREE_NODE_TYPE = 1;
//const int IMPORTTREE_TREE_LINE_NUMBER = 2;
      // keep in list of TreeNode
      synchronized(g_ddsTreeNodeData)
      {
        dynAppend(g_dsTreeNode, sNodeName);
//!!!        dynUnique(g_dsTreeNode);
        dsDataToKeep[IMPORTTREE_TREE_NODE_TYPE] = "_FwTreeNode";
        dsDataToKeep[IMPORTTREE_TREE_LINE_NUMBER] = iLineDevice;
        dsDataToKeep[IMPORTTREE_TREE_FILENAME] = sFileName;
        dynAppend(dsDataToKeep, strsplit(sDeviceData, ";"));
//!!!        g_mTreeDeviceData[sNodeName] = dsDataToKeep;
        g_ddsTreeNodeData[dynlen(g_dsTreeNode)] = dsDataToKeep;
      }
      sValue = unImportValidation_getTagValue(dsData, "<userdata>");
      dynClear(dsNavData);
      dsNavRebuilt = strsplit(sValue, "~");
      if(dynlen(dsNavRebuilt)>= 11)
      {
        dynRemove(dsNavRebuilt, 1);
        for(i=1;i<=10;i++)
          dsNavData[i] = "<link"+i+">"+dsNavRebuilt[i]+"</link"+i+">"; 
  		  iNavIndex = 1;
		    unImportValidation_devicenavigation_checkTagSectionData(unTreeWidget_treeNodeDpPrefix + sNodeName, dsNavData, 
                                                           bKeepLocalSystem, exInfo, iLineDevice, sFileName, bKeepInMemory);
        dynClear(dsNavData);
      }
    }
// if the device already exists, check if the parentnode name is the same, and device data is the same.
		strreplace(sNodeName, "@", "&");
		strreplace(sParentNodeName, "@", "&");
//DebugN(unTreeWidget_treeNodeDpPrefix+sNodeName, dpExists(unTreeWidget_treeNodeDpPrefix + sNodeName));
		if(dpExists(unTreeWidget_treeNodeDpPrefix + sNodeName)) {
			if(dpTypeName(unTreeWidget_treeNodeDpPrefix + sNodeName) == unTreeWidget_treeNodeDPT) {
				fwTree_getParent(sNodeName, sCurrentParentNodeName, exInfo);
				fwTree_getNodeDevice(sNodeName, sCurrentDeviceName, sCurrentDeviceType, exInfo);
				fwTree_getNodeCU(sNodeName, iCurrentNodeCU, exInfo);
				fwTree_getNodeUserData(sNodeName, dsUserData, exInfo);
				
//				if(!bKeepLocalSystem) {
				if(unExportTree_getSystemName(sDeviceName) == "#local:")
					sDeviceName = unExportTree_removeSystemName(sDeviceName);
				if(unExportTree_getSystemName(sCurrentDeviceName) == getSystemName())
					sCurrentDeviceName = unExportTree_removeSystemName(sCurrentDeviceName);
//				}
	// if no device in current node then no error
				if(sCurrentDeviceName == "") {
					sCurrentDeviceName = sDeviceName;
					sCurrentDeviceType = sDeviceType;
					iCurrentNodeCU = iNodeCU;
				}
	
				if(unImportValidation_getTagName(sTag) != sDeviceType) {
	//DebugN("ERROR", unImportValidation_getTagName(sTag), sDeviceType);
					fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: wrong device type in file, "+sNodeName, "");
				}
				
				if((sParentNodeName != sCurrentParentNodeName) || 
						(sDeviceName != sCurrentDeviceName) ||
						(sDeviceType != sCurrentDeviceType) ||
						(iNodeCU != iCurrentNodeCU) ) {
	//DebugN(sParentNodeName, sCurrentParentNodeName, sDeviceName, sCurrentDeviceName, sDeviceType, sCurrentDeviceType, iNodeCU, iCurrentNodeCU);
					fwException_raise(exInfo, "WARNING", "unImportValidation__FwTreeNode_checkTagSectionData: node exists but config different, "+sNodeName+
															", current:"+sCurrentParentNodeName+","+sCurrentDeviceName+","+sCurrentDeviceType+","+iCurrentNodeCU+
															"; file:"+sParentNodeName+","+sDeviceName+","+sDeviceType+","+iNodeCU, "");
				}
	// check user data
				len=dynlen(dsUserData);
// if more than 1 entry in dsUserData --> assume panel navigation
				sConf = "";
				for(i=1;i<=len;i++) {
// add the device to the list to export
					if(i>1) {
						dsSplit = strsplit(dsUserData[i], c_panel_navigation_delimiter);
						while(dynlen(dsSplit) <3)
							dynAppend(dsSplit, ""); 
						if(dsSplit[2] != "") {
//							if(!bKeepLocalSystem) {
							if(unExportTree_getSystemName(dsSplit[2]) == getSystemName())
								dsSplit[2] = "#local:"+unExportTree_removeSystemName(dsSplit[2]);
//							}
							strreplace(dsSplit[2], "&", "@");
						}
						dsUserData[i]=dsSplit[1]+c_panel_navigation_delimiter+dsSplit[2]+c_panel_navigation_delimiter+dsSplit[3];
					}
					if(dsUserData[i] == "") // if empty force it to * in order to have an entry
						dsUserData[i] = "*";
					if(sConf == "") 
						sConf = dsUserData[i];
					else
						sConf = sConf+"~"+dsUserData[i];
				}
				if(sConf != "")
					sConf+="~";
// if * then replace it by ""
				strreplace(sConf, "*", "");
				sValue = unImportValidation_getTagValue(dsData, "<userdata>");
				if(sConf != sValue)
					fwException_raise(exInfo, "WARNING", "unImportValidation__FwTreeNode_checkTagSectionData: node exists but user data config different, "+sNodeName+
																		", current:"+sConf+";file:"+sValue, "");
			}
			else
				fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_checkTagSectionData: dpnode exists but wrong type name, "+unTreeWidget_treeNodeDpPrefix + sNodeName, "");
		}
	}
//DebugN("+++++", sNodeName, sParentNodeName, exInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation__FwTreeNode_importTagSectionData
/**
Purpose:
Import the data of the treenode

	@param dsData: dyn_string, input: the data for the tag
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation__FwTreeNode_importTagSectionData(dyn_string dsData, dyn_string &exInfo)
{
	string sValue, sDpNodeName, sNodeName, sParentNodeName, sDeviceName, sDeviceType, sTag, sParentDpNodeName;
	dyn_string dsSplit, dsChildren, dsDevData, dsSplitBis;
	int iNodeCU, i, len, iDataIndex;
	
//DebugN("unImportValidation__FwTreeNode_importTagSectionData", dsData[1]);
// get the name and check it
	sValue = unImportValidation_getTagValue(dsData, "<name>");
// remove any @ and replace them by &
	strreplace(sValue, "@", "&");
	sNodeName = sValue;
	sDpNodeName = _fwTree_makeNodeName(sNodeName);
	if(!dpExists(sDpNodeName)) {
		unImportValidation_createDp(sDpNodeName,unTreeWidget_treeNodeDPT, exInfo);
//DebugN("unImportValidation__FwTreeNode_importTagSectionData, create Dp", sDpNodeName);
	}
	if(dynlen(exInfo) <= 0) {
		if(dpTypeName(sDpNodeName) == unTreeWidget_treeNodeDPT) {
// get the parent
			sValue = unImportValidation_getTagValue(dsData, "<parentname>");
// remove any @ and replace them by &
			strreplace(sValue, "@", "&");
			sParentNodeName = sValue;

// get the device name, device type and nodeCU and check them
			sValue = unImportValidation_getTagValue(dsData, "<devicedata>");
			dsSplit = strsplit(sValue, UN_PARAMETER_DELIMITER);
			sDeviceName = dsSplit[1];
			if(unExportTree_getSystemName(sDeviceName) == "#local:")
				sDeviceName = getSystemName()+unExportTree_removeSystemName(sDeviceName);
			sDeviceType = dsSplit[2];
			sscanf(dsSplit[3], "%d", iNodeCU);
// get the list of children
			sValue = unImportValidation_getTagValue(dsData, "<children>");
			dsSplit = strsplit(sValue, UN_PARAMETER_DELIMITER);
			len = dynlen(dsSplit);
			for(i=1;i<=len;i++) {
				strreplace(dsSplit[i], "@", "&");
				dsChildren[i] = dsSplit[i];
				if(dsChildren[i] != "")
					dsChildren[i] = _fwTree_makeNodeName(dsChildren[i]);
			}
// get the userdata
			sValue = unImportValidation_getTagValue(dsData, "<userdata>");
// replace any @ by &
			strreplace(sValue, "fwTN_@", "fwTN_&");
			dsSplit = strsplit(sValue, "~");
			len=dynlen(dsSplit);
// if more than 1 entry in dsSplit --> assume panel navigation
			for(i=1;i<=len;i++) {
// add the device to the list to export
				if(i>1) {
					dsSplitBis = strsplit(dsSplit[i], c_panel_navigation_delimiter);
					while(dynlen(dsSplitBis) <3)
						dynAppend(dsSplitBis, ""); 
					if(dsSplitBis[2] != "") {
						if(unExportTree_getSystemName(dsSplitBis[2]) == "#local:")
							dsSplitBis[2] = getSystemName()+unExportTree_removeSystemName(dsSplitBis[2]);
						strreplace(dsSplitBis[2], "&", "@");
					}
					dsSplit[i]=dsSplitBis[1]+c_panel_navigation_delimiter+dsSplitBis[2]+c_panel_navigation_delimiter+dsSplitBis[3];
				}
			}
// set the tree node device data.
			fwTree_setNodeDevice(sNodeName, sDeviceName, sDeviceType, exInfo);
			fwTree_setNodeUserData(sNodeName, dsSplit, exInfo);
			fwTree_setNodeCU(sNodeName, iNodeCU, exInfo);
			_fwTree_setChildren(sDpNodeName, dsChildren, exInfo);

// if parent exists, add it into parent if not already added, and set the parent
			fwTree_addNode(sParentNodeName, sNodeName, exInfo, 1);
			if(sParentNodeName == "") // case sParentNodeName == "" and sNodeName has a parent (bug fwTree.ctl)
				dpSet(sDpNodeName+".parent","");

// set the device data
			dsChildren = makeDynString();
			iDataIndex = 1;
			unImportValidation_getTagSectionData(dsData, dsChildren, iDataIndex, "<deviceconfiguration>", "</deviceconfiguration>");
			if(dynlen(dsChildren) > 0) { // there is a device so import it
				iDataIndex = 1;
				unImportValidation_getNextTagSectionData(dsChildren, dsDevData, iDataIndex, sTag);
//DebugN("unImportValidation__FwTreeNode_importTagSectionData set device config");
				unImportValidation_importTagSectionData(unImportValidation_getTagName(sTag), dsDevData, exInfo);
			}
		}
		else
			fwException_raise(exInfo, "ERROR", "unImportValidation__FwTreeNode_importTagSectionData: wrong device type , "+sDpNodeName, "");
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation__UnPanel_checkTagSectionData
/**
Purpose:
Check the data of the _UnPanel device

	@param bTreeNodeDevice: bool, input: true=the function is called from a treenode and the tree node device name is given
	@param sTreeNodeDeviceName: string, input: the tree node device name
	@param dsData: dyn_string, input: the data for the tag
	@param bKeepLocalSystem: bool, input, keep the local system name, do not replace it by #local
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation__UnPanel_checkTagSectionData(bool bTreeNodeDevice, string sTreeNodeDeviceName, dyn_string dsData, bool bKeepLocalSystem, 
                                           dyn_string &exInfo, int iLineDevice, string sFileName, bool bKeepInMemory)
{
	string sDpName, sValue, sTag, sPanelName;
	dyn_string dsNavData, dsDataToKeep;
	int iNavIndex;
  dyn_string dsNavRebuilt;
  int i;
  
//DebugN("unImportValidation__UnPanel_checkTagSectionData", dsData[1], iLineDevice, bKeepInMemory);	
// check the size: minimum 3
	if(dynlen(dsData) < 13) {
		fwException_raise(exInfo, "ERROR", "unImportValidation__UnPanel_checkTagSectionData: wrong data length", "");
	}
	else {
// get the panel name and check it
		sValue = unImportValidation_getTagValue(dsData, "<panelname>");
    sPanelName = sValue;
		sDpName = unPanel_convertPanelFileNameToDp(sValue);
		sDpName = unExportTree_removeSystemName(sDpName);
		sValue = sDpName;
		strreplace(sValue, "-", "");
//DebugN(sDpName, sValue, unImportValidation_getTagValue(dsData, "<panelname>"));
		if(!unImportValidation_checkNodeName(sValue) || (sValue == "")) {
			fwException_raise(exInfo, "ERROR", "unImportValidation__UnPanel_checkTagSectionData: wrong device dp name, "+sValue, "");
		}
		
// check that the tree node device name and panel file name are the same
		if(bTreeNodeDevice) {
			sValue = unExportTree_removeSystemName(sTreeNodeDeviceName);
			if(sValue != sDpName)
				fwException_raise(exInfo, "ERROR", "unImportValidation__UnPanel_checkTagSectionData: tree device node and panel file name are different, "+sValue+", "+sDpName, "");
		}
    if(bKeepInMemory)
    {
//global mapping g_mTreeDeviceData;
//global dyn_string g_dsTreeNode, g_dsPanel, g_dsPage, g_dsPlot, g_dsNavigation;
//const int IMPORTTREE_TREE_NODE_TYPE = 1;
//const int IMPORTTREE_TREE_LINE_NUMBER = 2;
      // keep in list of TreeNode
      synchronized(g_ddsTreeNodeData)
      {
        dynAppend(g_dsPanel, sPanelName);
//!!!        dynUnique(g_dsPanel);
        dsDataToKeep[IMPORTTREE_TREE_NODE_TYPE] = "_UnPanel";
        dsDataToKeep[IMPORTTREE_TREE_LINE_NUMBER] = iLineDevice;
        dsDataToKeep[IMPORTTREE_TREE_FILENAME] = sFileName;
        dynAppend(dsDataToKeep, sDpName);
//!!!        g_mTreeDeviceData[sPanelName] = dsDataToKeep;
        g_ddsPanelData[dynlen(g_dsPanel)] = dsDataToKeep;
      }
    }
// check the dptype if the dp exists
		if(dpExists(sDpName)) {
			if(dpTypeName(sDpName) != UN_WINDOWTREE_PANEL)
				fwException_raise(exInfo, "ERROR", "unImportValidation__UnPanel_checkTagSectionData: dpnode exists but wrong type name, "+sDpName, "");
		}
// get the horizontal navigation and check them
		iNavIndex = 1;
		unImportValidation_getTagSectionData(dsData, dsNavData, iNavIndex, "<devicenavigation>", "</devicenavigation>");
		unImportValidation_devicenavigation_checkTagSectionData(sDpName, dsNavData, bKeepLocalSystem, exInfo, iLineDevice, sFileName, bKeepInMemory);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation__UnPanel_importTagSectionData
/**
Purpose:
Import the data of the _UnPanel device

	@param dsData: dyn_string, input: the data for the tag
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation__UnPanel_importTagSectionData(dyn_string dsData, dyn_string &exInfo)
{
	string sValue, sDpName;
	int iNavIndex;
	dyn_string dsNavData;
	
//DebugN("unImportValidation__UnPanel_importTagSectionData", dsData[1]);
// get the panel name and check it
	sValue = unImportValidation_getTagValue(dsData, "<panelname>");
	sDpName = unPanel_convertPanelFileNameToDp(sValue);
	sDpName = unExportTree_removeSystemName(sDpName);
	if(!dpExists(sDpName)) {
		unImportValidation_createDp(sDpName, UN_WINDOWTREE_PANEL, exInfo);
//DebugN("unImportValidation__UnPanel_importTagSectionData, create Dp", sDpName);
	}
	if(dynlen(exInfo) <= 0) {
// get the horizontal navigation and check them
		iNavIndex = 1;
		unImportValidation_getTagSectionData(dsData, dsNavData, iNavIndex, "<devicenavigation>", "</devicenavigation>");
		unImportValidation_devicenavigation_importTagSectionData(sDpName, dsNavData, exInfo);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_FwTrendingPlot_checkTagSectionData
/**
Purpose:
Check the data of the FwTrendingPlot device

	@param bTreeNodeDevice: bool, input: true=the function is called from a treenode and the tree node device name is given
	@param sTreeNodeDeviceName: string, input: the tree node device name
	@param dsData: dyn_string, input: the data for the tag
	@param bKeepLocalSystem: bool, input, keep the local system name, do not replace it by #local
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation_FwTrendingPlot_checkTagSectionData(bool bTreeNodeDevice, string sTreeNodeDeviceName, dyn_string dsData, bool bKeepLocalSystem, 
                                                 dyn_string &exInfo, int iLineDevice, string sFileName, bool bKeepInMemory)
{
	string sDpName, sValue, sTag, sSystemName, sLegend;
	dyn_string dsNavData, dsSplit, dsDataToKeep;
	int iNavIndex;
	dyn_dyn_string ddsPlotData, ddsPlotDataFromFile;
	string sConf, sDpeConf;
	int i, len;
	bool bStop = false;

//DebugN("unImportValidation_FwTrendingPlot_checkTagSectionData", dsData[1], iLineDevice,  bKeepInMemory);
// check the size: minimum 10
	if(dynlen(dsData) < 24) {
    if(dynlen(dsData) >= 1)
      sConf = dsData[1];
		fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPlot_checkTagSectionData: wrong data length, "+ sConf+", len="+dynlen(dsData)+", should be 24", "");
	}
	else {
// get the dp name and check it
		sDpName = unImportValidation_getTagValue(dsData, "<dpname>");
		sSystemName = unExportTree_getSystemName(sDpName);
		if((sSystemName == "#local:") || (sSystemName == ""))
			sDpName = getSystemName()+unExportTree_removeSystemName(sDpName);
		sValue = unExportTree_removeSystemName(sDpName);
		strreplace(sValue, "-", "");
		if(!unImportValidation_checkNodeName(sValue) || (sValue == "")) {
			fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPlot_checkTagSectionData: wrong device dp name, "+sValue, "");
		}
// check that the tree node device name and plot dpname are the same
		if(bTreeNodeDevice) {
			sValue = sTreeNodeDeviceName;
			sSystemName = unExportTree_getSystemName(sTreeNodeDeviceName);
			if((sSystemName == "#local:") || (sSystemName == ""))
				sValue = getSystemName()+unExportTree_removeSystemName(sValue);

//DebugN("unImportValidation_FwTrendingPlot_checkTagSectionData", sValue, sDpName);
			if(sValue != sDpName)
				fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPlot_checkTagSectionData: tree device node and plot dpname are different, "+sValue+", "+sDpName, "");
		}
    // get the plot configuration from file
    unImportValidation_FwTrendingPlot_getFromFile(dsData, ddsPlotDataFromFile, dsNavData);
    if(bKeepInMemory)
    {
//global mapping g_mTreeDeviceData;
//global dyn_string g_dsTreeNode, g_dsPanel, g_dsPage, g_dsPlot, g_dsNavigation;
//const int IMPORTTREE_TREE_NODE_TYPE = 1;
//const int IMPORTTREE_TREE_LINE_NUMBER = 2;
      // keep in list of TreeNode
      synchronized(g_ddsTreeNodeData)
      {
        dynAppend(g_dsPlot, ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_TITLE][1]);
//!!!        dynUnique(g_dsPlot);
        dsDataToKeep[IMPORTTREE_TREE_NODE_TYPE] = "FwTrendingPlot";
        dsDataToKeep[IMPORTTREE_TREE_LINE_NUMBER] = iLineDevice;
        dsDataToKeep[IMPORTTREE_TREE_FILENAME] = sFileName;
        dynAppend(dsDataToKeep, ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_DPES]);
        dynAppend(dsDataToKeep, ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_DPES_X]);
//!!!        g_mTreeDeviceData[ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_TITLE][1]] = dsDataToKeep;
        g_ddsPlotData[dynlen(g_dsPlot)] = dsDataToKeep;
      }
    }
		if(!dpIsLegalName(ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_TITLE][1]) || (ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_TITLE][1] == "")) {
			fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPlot_checkTagSectionData: wrong plot title, "+ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_TITLE][1], "");
		}
// if the dp exists check the content 
		if(dpExists(sDpName)) {
			if(dpTypeName(sDpName) == UN_TRENDTREE_PLOT) {
// get the device data
				fwTrending_getPlot(sDpName, ddsPlotData, exInfo);
				if(ddsPlotData != ddsPlotDataFromFile)
					fwException_raise(exInfo, "WARNING", "unImportValidation_FwTrendingPlot_checkTagSectionData: different plot configuration, "+sDpName+
																			", current:"+ddsPlotData+";file"+ddsPlotDataFromFile, "");
//DebugN("dp plot", ddsPlotData);
			}
			else
				fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPlot_checkTagSectionData: dpnode exists but wrong type name, "+sDpName, "");
		}
// get the horizontal navigation and check them
		iNavIndex = 1;
		dsNavData = makeDynString();
		unImportValidation_getTagSectionData(dsData, dsNavData, iNavIndex, "<devicenavigation>", "</devicenavigation>");
//DebugN(sDpName, dsData, dsNavData, iNavIndex, "end");
		unImportValidation_devicenavigation_checkTagSectionData(sDpName, dsNavData, bKeepLocalSystem, exInfo, iLineDevice, sFileName, bKeepInMemory);
	}
//  DebugN("plot", exInfo);
}
unImportValidation_FwTrendingPlot_getFromFile(dyn_string dsData, dyn_dyn_string &plotData, dyn_string &dsNavData)
{
  int iIndex;
  dyn_string dsValue, dsPlotConf, dsPlotTemplate;
  string sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<model>");
  plotData[fwTrending_PLOT_OBJECT_MODEL][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<plotTitle>");
  plotData[fwTrending_PLOT_OBJECT_TITLE][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<legend>");
  plotData[fwTrending_PLOT_OBJECT_LEGEND_ON][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<plotBackColor>");
  plotData[fwTrending_PLOT_OBJECT_BACK_COLOR][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<plotForeColor>");
  plotData[fwTrending_PLOT_OBJECT_FORE_COLOR][1] = sValue;
  
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<dpes>", "</dpes>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<dpes>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_DPES] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<xDpes>", "</xDpes>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<xDpes>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_DPES_X] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<legendTexts>", "</legendTexts>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<legendTexts>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_LEGENDS] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<xLegendTexts>", "</xLegendTexts>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<xLegendTexts>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_LEGENDS_X] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<colors>", "</colors>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<colors>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_COLORS] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<axii>", "</axii>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<axii>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_AXII] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<xAxii>", "</xAxii>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<xAxii>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_AXII_X] = dsPlotConf;
  
  sValue = unImportValidation_getTagValue(dsData, "<isTemplate>");
  plotData[fwTrending_PLOT_OBJECT_IS_TEMPLATE][1] = sValue;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<curvesHidden>", "</curvesHidden>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<curvesHidden>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_CURVES_HIDDEN] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<yRangeMin>", "</yRangeMin>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<yRangeMin>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_RANGES_MIN] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<yRangeMax>", "</yRangeMax>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<yRangeMax>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_RANGES_MAX] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<xRangeMin>", "</xRangeMin>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<xRangeMin>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_RANGES_MIN_X] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<xRangeMax>", "</xRangeMax>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<xRangeMax>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_RANGES_MAX_X] = dsPlotConf;
  
  sValue = unImportValidation_getTagValue(dsData, "<plotType>");
  plotData[fwTrending_PLOT_OBJECT_TYPE][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<timeRange>");
  plotData[fwTrending_PLOT_OBJECT_TIME_RANGE][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<templateName>");
  plotData[fwTrending_PLOT_OBJECT_TEMPLATE_NAME][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<isLogarithmic>");
  plotData[fwTrending_PLOT_OBJECT_IS_LOGARITHMIC][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<grid>");
  plotData[fwTrending_PLOT_OBJECT_GRID][1] = sValue;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<curveTypes>", "</curveTypes>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<curveTypes>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_CURVE_TYPES] = dsPlotConf;
  
  sValue = unImportValidation_getTagValue(dsData, "<markerType>");
  plotData[fwTrending_PLOT_OBJECT_MARKER_TYPE][1] = sValue;
  
  plotData[fwTrending_PLOT_OBJECT_ACCESS_CONTROL_SAVE][1] = "";
  
  sValue = unImportValidation_getTagValue(dsData, "<controlBar>");
  plotData[fwTrending_PLOT_OBJECT_CONTROL_BAR_ON][1] = sValue;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<axiiPos>", "</axiiPos>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<axiiPos>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_AXII_POS] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<axiiLink>", "</axiiLink>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<axiiLink>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_AXII_LINK] = dsPlotConf;
  
  sValue = unImportValidation_getTagValue(dsData, "<defaultFont>");
  plotData[fwTrending_PLOT_OBJECT_DEFAULT_FONT][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<curveStyle>");
  plotData[fwTrending_PLOT_OBJECT_CURVE_STYLE][1] = sValue;
  
  sValue = unImportValidation_getTagValue(dsData, "<curveStyle>");
  plotData[fwTrending_PLOT_OBJECT_CURVE_STYLE][1] = sValue;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<xAxiiFormat>", "</xAxiiFormat>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<xAxiiFormat>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_AXII_X_FORMAT] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<legendValuesFormat>", "</legendValuesFormat>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<legendValuesFormat>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_LEGEND_VALUES_FORMAT] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<yAxiiFormat>", "</yAxiiFormat>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<yAxiiFormat>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_AXII_Y_FORMAT] = dsPlotConf;
  
  dynClear(dsPlotConf);
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<alarmLimitsVisible>", "</alarmLimitsVisible>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<alarmLimitsVisible>");
    dynAppend(dsPlotConf, sValue);
    dynRemove(dsValue, 1);
  }
  plotData[fwTrending_PLOT_OBJECT_ALARM_LIMITS_SHOW] = dsPlotConf;
  
//  DebugN("plot file", plotData);
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_FwTrendingPlot_importTagSectionData
/**
Purpose:
Check the data of the FwTrendingPlot device

	@param bTreeNodeDevice: bool, input: true=the function is called from a treenode and the tree node device name is given
	@param sTreeNodeDeviceName: string, input: the tree node device name
	@param dsData: dyn_string, input: the data for the tag
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation_FwTrendingPlot_importTagSectionData(dyn_string dsData, dyn_string &exInfo)
{
	string sDpName, sSystemName;
	dyn_string dsNavData;
	int iNavIndex;
	dyn_dyn_string ddsPlotDataFromFile;
	
//DebugN("unImportValidation_FwTrendingPlot_importTagSectionData", dsData[1]);
// get the dp name and check it
	sDpName = unImportValidation_getTagValue(dsData, "<dpname>");
	sSystemName = unExportTree_getSystemName(sDpName);
	if(sSystemName == "#local:")
		sDpName = unExportTree_removeSystemName(sDpName);
	if(!dpExists(sDpName)) {
		unImportValidation_createDp(sDpName, UN_TRENDTREE_PLOT, exInfo);
//DebugN("unImportValidation_FwTrendingPlot_importTagSectionData, create Dp", sDpName);
	}
	if(dynlen(exInfo) <= 0) {
    // get the plot configuration from file
    unImportValidation_FwTrendingPlot_getFromFile(dsData, ddsPlotDataFromFile, dsNavData);
//    DebugN(sDpName);
    unImportValidation_FwTrendingPlot_addMissingReplaceSystemName(ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_DPES_X]);
    unImportValidation_FwTrendingPlot_addMissingReplaceSystemName(ddsPlotDataFromFile[fwTrending_PLOT_OBJECT_DPES]);
		fwTrending_setPlot(sDpName, ddsPlotDataFromFile, exInfo);

// get the horizontal navigation and check them
		iNavIndex = 1;
		dsNavData = makeDynString();
		unImportValidation_getTagSectionData(dsData, dsNavData, iNavIndex, "<devicenavigation>", "</devicenavigation>");
		unImportValidation_devicenavigation_importTagSectionData(sDpName, dsNavData, exInfo);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_FwTrendingPlot_addMissingReplaceSystemName
/**
Purpose:
Add the system name in the FwTrendingPlot DPE adn xDPE data if missing: ENS-12882

	@param dsDPE: dyn_string, FwTrendingPlot DPE config data

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_FwTrendingPlot_addMissingReplaceSystemName(dyn_string &dsDPE)
{
  int i, len;
  
  len = dynlen(dsDPE);
//  DebugN("bef", dsDPE);
  for(i=1;i<=len;i++)
  {
    if((dsDPE[i] != "") && (strpos(dsDPE[i], ":") <0))
      dsDPE[i] = getSystemName()+dsDPE[i];
    else if(unExportTree_getSystemName(dsDPE[i]) == "#local:")
      strreplace(dsDPE[i], "#local:", getSystemName());
  }
//  DebugN("end", dsDPE);
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_FwTrendingPage_checkTagSectionData
/**
Purpose:
Check the data of the FwTrendingPage device

	@param bTreeNodeDevice: bool, input: true=the function is called from a treenode and the tree node device name is given
	@param sTreeNodeDeviceName: string, input: the tree node device name
	@param dsData: dyn_string, input: the data for the tag
	@param bKeepLocalSystem: bool, input, keep the local system name, do not replace it by #local
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation_FwTrendingPage_checkTagSectionData(bool bTreeNodeDevice, string sTreeNodeDeviceName, dyn_string dsData, bool bKeepLocalSystem, 
                                                 dyn_string &exInfo, int iLineDevice, string sFileName, bool bKeepInMemory)
{
	string sDpName, sValue, sTag, sSystemName, sConf;
	dyn_string dsNavData, dsSplit, dsDataToKeep;
	int iNavIndex, i, len;
	dyn_dyn_string ddsPageData, ddsPageDataFromFile;
	bool bStop=false;
	
//DebugN("unImportValidation_FwTrendingPage_checkTagSectionData", dsData[1], iLineDevice, bKeepInMemory);
// check the size: minimum 3 plus list of plot depending on row and col
	if(dynlen(dsData) < 17) {
    if(dynlen(dsData) >= 1)
      sConf = dsData[1];
		fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPage_checkTagSectionData: wrong data length, "+ sConf+", len="+dynlen(dsData)+", should be 17", "");
	}
	else {
// get the dp name and check it
		sDpName = unImportValidation_getTagValue(dsData, "<dpname>");
		sSystemName = unExportTree_getSystemName(sDpName);
		if((sSystemName == "#local:") || (sSystemName == ""))
			sDpName = getSystemName()+unExportTree_removeSystemName(sDpName);
		sValue = unExportTree_removeSystemName(sDpName);
		strreplace(sValue, "-", "");
		if(!unImportValidation_checkNodeName(sValue) || (sValue == "")) {
			fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPage_checkTagSectionData: wrong device dp name, "+sValue, "");
		}
// check that the tree node device name and page dpname are the same
		if(bTreeNodeDevice) {
			sValue = sTreeNodeDeviceName;
			sSystemName = unExportTree_getSystemName(sTreeNodeDeviceName);
			if((sSystemName == "#local:") || (sSystemName == ""))
				sValue = getSystemName()+unExportTree_removeSystemName(sValue);
	
	//DebugN("unImportValidation_FwTrendingPage_checkTagSectionData", sValue, sDpName);
			if(sValue != sDpName)
				fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPage_checkTagSectionData: tree device node and page dpname are different, "+sValue+", "+sDpName, "");
		}
    // get the page configuration from file
    unImportValidation_FwTrendingPage_getFromFile(dsData, ddsPageDataFromFile, dsNavData);
    if(bKeepInMemory)
    {
//global mapping g_mTreeDeviceData;
//global dyn_string g_dsTreeNode, g_dsPanel, g_dsPage, g_dsPlot, g_dsNavigation;
//const int IMPORTTREE_TREE_NODE_TYPE = 1;
//const int IMPORTTREE_TREE_LINE_NUMBER = 2;
      // keep in list of TreeNode
      synchronized(g_ddsTreeNodeData)
      {
        dynAppend(g_dsPage, ddsPageDataFromFile[fwTrending_PAGE_OBJECT_TITLE][1]);
//!!!        dynUnique(g_dsPage);
        dsDataToKeep[IMPORTTREE_TREE_NODE_TYPE] = "FwTrendingPage";
        dsDataToKeep[IMPORTTREE_TREE_LINE_NUMBER] = iLineDevice;
        dsDataToKeep[IMPORTTREE_TREE_FILENAME] = sFileName;
        dynAppend(dsDataToKeep, ddsPageDataFromFile[fwTrending_PAGE_OBJECT_PLOTS]);
//!!!        g_mTreeDeviceData[ddsPageDataFromFile[fwTrending_PAGE_OBJECT_TITLE][1]] = dsDataToKeep;
        g_ddsPageData[dynlen(g_dsPage)] = dsDataToKeep;
      }
    }
		if(!dpIsLegalName(ddsPageDataFromFile[fwTrending_PAGE_OBJECT_TITLE][1]) || (ddsPageDataFromFile[fwTrending_PAGE_OBJECT_TITLE][1] == "")) {
			fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPage_checkTagSectionData: wrong page title, "+ddsPageDataFromFile[fwTrending_PAGE_OBJECT_TITLE][1], "");
		}
// if the dp exists check the content 
		if(dpExists(sDpName)) {
			if(dpTypeName(sDpName) == UN_TRENDTREE_PAGE) {
// get the device data
				fwTrending_getPage(sDpName, ddsPageData, exInfo);
        
				if(ddsPageData != ddsPageDataFromFile)
					fwException_raise(exInfo, "WARNING", "unImportValidation_FwTrendingPage_checkTagSectionData: different page configuration, "+sDpName+
																			", current:"+ddsPageData+";file"+ddsPageDataFromFile, "");
//DebugN("dp page", ddsPageData);
			}
			else
				fwException_raise(exInfo, "ERROR", "unImportValidation_FwTrendingPage_checkTagSectionData: dpnode exists but wrong type name, "+sDpName, "");
		}
// get the horizontal navigation and check them
		iNavIndex = 1;
		dsNavData = makeDynString();
		unImportValidation_getTagSectionData(dsData, dsNavData, iNavIndex, "<devicenavigation>", "</devicenavigation>");
		unImportValidation_devicenavigation_checkTagSectionData(sDpName, dsNavData, bKeepLocalSystem, exInfo, iLineDevice, sFileName, bKeepInMemory);
	}
//  DebugN("page", exInfo);
}
unImportValidation_FwTrendingPage_getFromFile(dyn_string dsData, dyn_dyn_string &ddsPageData, dyn_string &dsNavData)
{
  int iIndex;
  dyn_string dsValue, dsPlot, dsPlotTemplate;
  string sTitle, sModel, nCol, nRow, sControl, sValue;
  
  sTitle = unImportValidation_getTagValue(dsData, "<pageTitle>");
  sModel = unImportValidation_getTagValue(dsData, "<model>");
  nCol = unImportValidation_getTagValue(dsData, "<nColumns>");
  nRow = unImportValidation_getTagValue(dsData, "<nRows>");
  sControl = unImportValidation_getTagValue(dsData, "<controls>");
  ddsPageData[fwTrending_PAGE_OBJECT_MODEL][1] = sModel;
  ddsPageData[fwTrending_PAGE_OBJECT_TITLE][1] = sTitle;
  ddsPageData[fwTrending_PAGE_OBJECT_NCOLS][1] = nCol;
  ddsPageData[fwTrending_PAGE_OBJECT_NROWS][1] = nRow;
  ddsPageData[fwTrending_PAGE_OBJECT_CONTROLS][1] = sControl;
  ddsPageData[fwTrending_PAGE_OBJECT_ACCESS_CONTROL_SAVE][1] = "";
  
  iIndex = 1;
	unImportValidation_getTagSectionData(dsData, dsNavData, iIndex, "<devicenavigation>", "</devicenavigation>");
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<plots>", "</plots>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<plots>");
    dynAppend(dsPlot, sValue);
    dynRemove(dsValue, 1);
  }
  ddsPageData[fwTrending_PAGE_OBJECT_PLOTS] = dsPlot;
  dynClear(dsValue);
  iIndex = 1;
  unImportValidation_getTagSectionData(dsData, dsValue, iIndex, "<plotTemplateParams>", "</plotTemplateParams>");
  while(dynlen(dsValue) >= 1)
  {
    sValue = unImportValidation_getTagValue(dsValue, "<plotTemplateParams>");
    dynAppend(dsPlotTemplate, sValue);
    dynRemove(dsValue, 1);
  }
  ddsPageData[fwTrending_PAGE_OBJECT_PLOT_TEMPLATE_PARAMETERS] = dsPlotTemplate;
  
//  DebugN("page file", ddsPageData);
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_FwTrendingPage_importTagSectionData
/**
Purpose:
Check the data of the FwTrendingPage device

	@param dsData: dyn_string, input: the data for the tag
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
  
  @reviewed 2018-06-22 @whitelisted{UNICOSTreeImport}
*/
unImportValidation_FwTrendingPage_importTagSectionData(dyn_string dsData, dyn_string &exInfo)
{
	string sDpName, sSystemName;
	dyn_string dsNavData;
	int iNavIndex;
	dyn_dyn_string ddsPageData, ddsPageDataFromFile;
	
//DebugN("unImportValidation_FwTrendingPage_importTagSectionData", dsData[1]);
// get the dp name and check it
	sDpName = unImportValidation_getTagValue(dsData, "<dpname>");
	sSystemName = unExportTree_getSystemName(sDpName);
	if(sSystemName == "#local:")
		sDpName = unExportTree_removeSystemName(sDpName);
	if(!dpExists(sDpName)) {
		unImportValidation_createDp(sDpName, UN_TRENDTREE_PAGE, exInfo);
//DebugN("unImportValidation_FwTrendingPlot_importTagSectionData, create Dp", sDpName);
	}
	if(dynlen(exInfo) <= 0) {
    // get the page configuration from file
    unImportValidation_FwTrendingPage_getFromFile(dsData, ddsPageDataFromFile, dsNavData);
		fwTrending_setPage(sDpName, ddsPageDataFromFile, exInfo);
		
// get the horizontal navigation and check them
		iNavIndex = 1;
		dsNavData = makeDynString();
		unImportValidation_getTagSectionData(dsData, dsNavData, iNavIndex, "<devicenavigation>", "</devicenavigation>");
		unImportValidation_devicenavigation_importTagSectionData(sDpName, dsNavData, exInfo);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_devicenavigation_checkTagSectionData
/**
Purpose:
Check the device navigation

	@param sDpName: string, input: the data point name
	@param dsData: dyn_string, input: the data for the tag
	@param bKeepLocalSystem: bool, input, keep the local system name, do not replace it by #local
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_devicenavigation_checkTagSectionData(string sDpName, dyn_string dsData, bool bKeepLocalSystem, dyn_string &exInfo, 
                                                   int iLineDevice, string sFileName, bool bKeepInMemory)
{
	int i;
	string sLink, sXMLName, sTagName, sValue, sConf, sPanelName;
	dyn_string dsSplit,dsDataToKeep;
	dyn_string panelLabel, panelNameList, panelDollarParam;
	
//DebugN("unImportValidation_devicenavigation_checkTagSectionData", dsData[1], iLineDevice, bKeepInMemory, dynlen(dsData));
  // check the size: 10
	if(dynlen(dsData) < 10) {
		fwException_raise(exInfo, "ERROR", "unImportValidation_devicenavigation_checkTagSectionData: wrong data length", "");
	}
	else {
// if the device is on exists and the links are different --> WARNING
		if(dpExists(sDpName)) {
			unPanel_getPanelList(sDpName, panelLabel, panelNameList, panelDollarParam, exInfo);

			for(i=1; i<=c_unPanelConfiguration_maxPanelNavigation; i++) {
				sprintf(sLink, "link%d", i);
				sXMLName = panelNameList[i];
//				if(!bKeepLocalSystem) {
				if(unExportTree_getSystemName(sXMLName) == getSystemName())
					sXMLName = "#local:"+unExportTree_removeSystemName(sXMLName);
//				}
		// remove any &, & are not allowed in the xml file and replace them by @
				strreplace(sXMLName, "&", "@");
				sConf = panelLabel[i]+c_panel_navigation_delimiter+sXMLName+c_panel_navigation_delimiter+panelDollarParam[i];
				sValue = unImportValidation_getTagValue(dsData, sLink);
				sTagName = unImportValidation_getTagName(dsData[i]);
				if(sValue == "")
					sValue = "||";
//DebugN(sDpName, sTagName, sLink, sConf, sValue);
				if(sConf != sValue) {
						fwException_raise(exInfo, "WARNING", "unImportValidation_devicenavigation_checkTagSectionData: "+sDpName+" "+sLink+" value is different, "+sConf+", "+sValue, "");
				}
			}
		}
		else {
			sConf = unExportTree_getSystemName(sDpName);
			if(sConf == "")
				sConf = getSystemName();
			if(sConf != getSystemName())
				fwException_raise(exInfo, "ERROR", "unImportValidation_devicenavigation_checkTagSectionData: dp on remote system and not found, "+sDpName, "");
		}
		for(i=1;i<=10;i++) {
			sprintf(sLink, "link%d", i);
			sTagName = unImportValidation_getTagName(dsData[i]);
//      if(bKeepInMemory)
//        dynAppend(dsDataToKeep, sTagName);
			if(sTagName != sLink)
				fwException_raise(exInfo, "ERROR", "unImportValidation_devicenavigation_checkTagSectionData: "+sDpName+"wrong or missing "+sLink+" tag name, "+sTagName, "");
		}
    if(bKeepInMemory)
    {
  //global mapping g_mTreeDeviceData;
  //global dyn_string g_dsTreeNode, g_dsPanel, g_dsPage, g_dsPlot, g_dsNavigation;
  //const int IMPORTTREE_TREE_NODE_TYPE = 1;
  //const int IMPORTTREE_TREE_LINE_NUMBER = 2;
      // keep in list of TreeNode
      synchronized(g_ddsTreeNodeData)
      {
        for(i=1; i<=c_unPanelConfiguration_maxPanelNavigation; i++)
        {
          sprintf(sLink, "link%d", i);
          sValue = unImportValidation_getTagValue(dsData, sLink);
          dsSplit = strsplit(sValue, c_panel_navigation_delimiter);
          if(dynlen(dsSplit) >=2)
          {
            if(dsSplit[2] != "")
            {
              dynAppend(g_dsNavigation, dsSplit[2]);
//!!!              dynUnique(g_dsNavigation);
              dsDataToKeep[IMPORTTREE_TREE_NODE_TYPE] = "deviceNavigation";
              dsDataToKeep[IMPORTTREE_TREE_LINE_NUMBER] = iLineDevice;
              dsDataToKeep[IMPORTTREE_TREE_FILENAME] = sFileName;
//!!!              g_mDeviceNavigationData[dsSplit[2]] = dsDataToKeep;
              g_ddsNavigationData[dynlen(g_dsNavigation)] = dsDataToKeep;
            }
          }
        }
      }
    }
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_devicenavigation_importTagSectionData
/**
Purpose:
Check the device navigation

	@param sDpName: string, input: the data point name
	@param dsData: dyn_string, input: the data for the tag
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_devicenavigation_importTagSectionData(string sDpName, dyn_string dsData, dyn_string &exInfo)
{
	int i;
	string sLink, sValue, sXML;
	dyn_string dsSplit;
	dyn_string panelLabel, panelNameList, panelDollarParam;

//DebugN("unImportValidation_devicenavigation_importTagSectionData", dsData[1]);
	if(dpExists(sDpName)) {
		for(i=1; i<=c_unPanelConfiguration_maxPanelNavigation; i++) {
			sprintf(sLink, "link%d", i);
			sValue = unImportValidation_getTagValue(dsData, sLink);
			if(sValue == "")
				sValue = "||";
	
			dsSplit = strsplit(sValue, "|");
			while(dynlen(dsSplit)<3)
				dynAppend(dsSplit, "");
			panelLabel[i] = dsSplit[1];
			panelNameList[i] = dsSplit[2];
			panelDollarParam[i] = dsSplit[3];
	
			sXML = panelNameList[i];
			if(unExportTree_getSystemName(sXML) == "#local:")
				sXML = getSystemName()+unExportTree_removeSystemName(sXML);
	// replace any @ by &
			strreplace(sXML, "@", "&");
			panelNameList[i] = sXML;
		}
		unPanel_setPanelList(sDpName, panelLabel, panelNameList, panelDollarParam, exInfo);
//sValue = panelNameList;
//sLink = panelLabel;
//sXML = panelDollarParam;
//DebugN("unImportValidation_devicenavigation_importTagSectionData set panel", sLink, sValue, sXML);
	}
	else
		fwException_raise(exInfo, "ERROR", "unImportValidation_devicenavigation_importTagSectionData: dp does not exists "+sDpName, "");
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_checkTreeData
/**
Purpose:
Check the configuration data of the tree

	@param dsData: dyn_string, input: the data of the tree
	@param sTreeName: string, output: the tree node name
	@param bKeepLocalSystem: bool, input, keep the local system name, do not replace it by #local
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_checkTreeData(dyn_string dsData, string &sTreeName, bool bKeepLocalSystem, dyn_string &exInfo)
{
	string sValue, root, nodeTypes, nodeNames, sConf;
	int refresh, i, len;
	dyn_string dsMode;
	string sTreeDp;
	
	if(dynlen(dsData) != 3) {
		fwException_raise(exInfo, "ERROR", "unImportValidation_checkTreeData: wrong data length", "");
    DebugN("error", dynlen(dsData));
	}
	else {
// get tree name and check it
		sValue = unImportValidation_getTagValue(dsData, "<name>");
//DebugN(dsData[1], sValue);
		if(sValue != "") {
			if(!unImportValidation_checkNodeName(sValue) || (sValue == "")) {
				fwException_raise(exInfo, "ERROR", "unImportValidation_checkTreeData: wrong tree node name, "+sValue, "");
			}
// remove any @ are not allowed in the xml file and replace them by &
			strreplace(sValue, "@", "&");
			sTreeName = sValue;
			sTreeDp = unTreeWidget_treeTypeDpPrefix+sTreeName;
			if(dpExists(sTreeDp)) {
// get the tree config and compare them with the given, if different raise warning
				dpGet(sTreeDp+".name", sTreeName, sTreeDp+".root", root, sTreeDp+".nodeTypes", nodeTypes, sTreeDp+".nodeNames", nodeNames,
							sTreeDp+".refresh", refresh, sTreeDp+".modes", dsMode);
		// if the root is the current system name replace it by #local
//				if(!bKeepLocalSystem) {
				if(root == getSystemName())
					root = "#local:";
//				}
				sConf = root+UN_PARAMETER_DELIMITER+nodeTypes+UN_PARAMETER_DELIMITER+
									nodeNames+UN_PARAMETER_DELIMITER+refresh+UN_PARAMETER_DELIMITER;
				sValue = unImportValidation_getTagValue(dsData, "<treeparameters>");
				if(sValue != sConf)
					fwException_raise(exInfo, "WARNING", "unImportValidation_checkTreeData: different treeparameters, "+sValue+", "+sConf, "");

				sConf = "";
				len = dynlen(dsMode);
				for(i=1;i<=len;i++) {
					if(sConf == "") 
						sConf = dsMode[i];
					else
						sConf = sConf+UN_PARAMETER_DELIMITER+dsMode[i];
				}
				if(sConf != "")
					sConf += UN_PARAMETER_DELIMITER;
				sValue = unImportValidation_getTagValue(dsData, "<treemodes>");
				if(sValue != sConf)
					fwException_raise(exInfo, "WARNING", "unImportValidation_checkTreeData: different treemodes, "+sValue+", "+sConf, "");
			}
		}
		else
			fwException_raise(exInfo, "WARNING", "unImportValidation_checkTreeData: no tree definition, tree can be imported into a position", "");
	}
//    DebugN("start",dsData, sTreeName);
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_importTreeData
/**
Purpose:
Import the configuration data of the tree

	@param dsData: dyn_string, input: the data of the tree
	@param exInfo: dyn_string, output: errors are returned here

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_importTreeData(dyn_string dsData, dyn_string &exInfo)
{
	string sValue, root, nodeTypes, nodeNames, sConf;
	int refresh, i, len;
	dyn_string dsMode;
	string sTreeDp, sTreeName;
	dyn_string dsSplit;

//DebugN("unImportValidation_importTreeData");
// get tree name
	sValue = unImportValidation_getTagValue(dsData, "<name>");
	if(sValue != "") {
// remove any @ are not allowed in the xml file and replace them by &
		strreplace(sValue, "@", "&");
		sTreeName = sValue;
		sTreeDp = unTreeWidget_treeTypeDpPrefix+sTreeName;
		if(!dpExists(sTreeDp)) {
			unImportValidation_createDp(sTreeDp,"_FwTreeType", exInfo);
//DebugN("unImportValidation_importTreeData, create Dp", sTreeDp);
		}
		if(dynlen(exInfo) <= 0) {
			if(dpTypeName(sTreeDp) == "_FwTreeType") {
	// get the treeparameters
				sValue = unImportValidation_getTagValue(dsData, "<treeparameters>");
				dsSplit = strsplit(sValue, UN_PARAMETER_DELIMITER);
				root = dsSplit[1];
			// if the root is #local replace it by the current system name
				if(root == "#local:")
					root = getSystemName();
				nodeTypes = dsSplit[2];
				nodeNames = dsSplit[3];
				sscanf(dsSplit[4], "%d", refresh);
	// get the treeparameters
				sValue = unImportValidation_getTagValue(dsData, "<treemodes>");
				dsMode = strsplit(sValue, UN_PARAMETER_DELIMITER);
	// get the tree config and compare them with the given, if different raise warning
				dpSet(sTreeDp+".name", sTreeName, sTreeDp+".root", root, sTreeDp+".nodeTypes", nodeTypes, sTreeDp+".nodeNames", nodeNames,
							sTreeDp+".refresh", refresh, sTreeDp+".modes", dsMode);
			}
			else
				fwException_raise(exInfo, "WARNING", "unImportValidation_importTreeData: wrong device type, "+sTreeDp, "");
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_getTagSectionData
/**
Purpose:
Get all the data between sStartTag and sEndTag

	@param dsFileContent: dyn_string, input: the content of the file to check
	@param dsData: dyn_string, output: the data of the tree are returned here
	@param iIndex: int, output: the index in the file after extracting the data
	@param sStartTag: string, input: the start tag 
	@param sEndTag: string, input: the end tag 
	
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_getTagSectionData(dyn_string dsFileContent, dyn_string &dsData, int &iIndex, string sStartTag, string sEndTag)
{
	int i, j, len=dynlen(dsFileContent);
	bool bNotFound=true;

//  DebugN(iIndex, len, sStartTag, sEndTag);
// search the startTag tag from iIndex
	for(i=iIndex;(i<=len) && bNotFound;i++) {
		if(dsFileContent[i] == sStartTag) {
// found then get all the data until the first endTage and increment iIndex
			iIndex++;
//DebugN("found", i);
			for(j=(i+1);(j<=len) && bNotFound;j++) {
//DebugN("get", j, dsFileContent[j]);
				if(dsFileContent[j] != sEndTag) {
					dynAppend(dsData, dsFileContent[j]);
					iIndex++;
				}
				else {
					bNotFound=false;
				}
			}
			if(bNotFound)
				iIndex++;
		}
	}
//DebugN("bnotfound", bNotFound, iIndex);
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_getNextTagSectionData
/**
Purpose:
Get next tag data section

	@param dsFileContent: dyn_string, input: the content of the file to check
	@param dsData: dyn_string, output: the data of the tree are returned here
	@param iIndex: int, output: the index in the file after extracting the data
	@param sTag: string, output: the tag found with < and >
	
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unImportValidation_getNextTagSectionData(dyn_string dsFileContent, dyn_string &dsData, int &iIndex, string &sTag)
{
	bool bFound=false;
	dyn_string dsSplit;
	int len = dynlen(dsFileContent);
	string sTempStartTag, sTempEndTag;
	
	while(!bFound && (iIndex <= len)) {
//DebugN(dsFileContent[iIndex], strpos(dsFileContent[iIndex], ">"), strlen(dsFileContent[iIndex]), strpos(dsFileContent[iIndex], "<"), strpos(dsFileContent[iIndex], "/"));
//return;
		if(((strpos(dsFileContent[iIndex], ">")+1) == strlen(dsFileContent[iIndex])) &&
				(strpos(dsFileContent[iIndex], "<") == 0) &&
				(strpos(dsFileContent[iIndex], "/") < 0)
			) {
// this is a tag
//DebugN("tag", iIndex, dsFileContent[iIndex]);
				bFound = true;
				sTempStartTag = dsFileContent[iIndex];
		}
		else
			iIndex++;
	}
	if(bFound) {
		unImportValidation_getTagSectionData(dsFileContent, dsData, iIndex, sTempStartTag, "</"+unImportValidation_getTagName(sTempStartTag)+">");
		sTag = sTempStartTag;
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_checkNodeName
/**
Purpose:
Check if the node name is correct: true/false=correct/not correct

	@param sNodeName: string, input: the node name to check

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
bool unImportValidation_checkNodeName(string sNodeName)
{
	string sTempNode=sNodeName;

// replace any @ by ""
	strreplace(sTempNode, "@", "");
//DebugN("unImportValidation_checkNodeName", sNodeName, sTempNode);
	if(strpos(sNodeName,"---Clipboard") == 0) // clipboard so assume it is Ok
		return true;
	return(unConfigGenericFunctions_nameCheck(sTempNode));	
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_checkDpName
/**
Purpose:
Check if the dpname is correct: true/false=correct/not correct

	@param sDpName: string, input: the datapoint name to check

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
bool unImportValidation_checkDpName(string sDpName)
{
	bool bResult = true;
	string sTemp=sDpName;
	string sSystemName = unExportTree_getSystemName(sDpName);
	
	if((sSystemName == "#local:") || (sSystemName == getSystemName())) {
		strreplace(sTemp, "-", "");
		bResult = unImportValidation_checkNodeName(unExportTree_removeSystemName(sTemp));
	}
	return bResult;
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_getTagValue
/**
Purpose:
Get the value of a given tag, tag and /tag must be in the same line

	@param dsData: dyn_string, input: the data
	@param sTag: string, input: the tag to find into the data
	
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
string unImportValidation_getTagValue(dyn_string dsData, string sTag)
{
	int i, len = dynlen(dsData);
	string sTemp;
	
	for(i=1;i<=len;i++){
		if(strpos(dsData[i], sTag)>=0) { // tag found get value
			sTemp = substr(dsData[i], strpos(dsData[i], ">")+ 1, strlen(dsData[i]));
//DebugN(sTag, dsData[i], sTemp, strpos(sTemp, "</"));
			sTemp = substr(sTemp, 0, strpos(sTemp, "</"));
//DebugN("after", sTag, dsData[i], sTemp);
			i = len+1;
		}
	}
//DebugN(sTemp, strpos(sTemp, "<"));	
	return sTemp;
}

//------------------------------------------------------------------------------------------------------------------------
// unImportValidation_getTagName
/**
Purpose:
Get the tag name (the first text between < and > in a given string)

	@param sLine: string, input: the data
	
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
string unImportValidation_getTagName(string sLine)
{
	string sValue = substr(sLine, strpos(sLine, "<")+1, strpos(sLine, ">")-1);
//DebugN("tagName", sLine, sValue, strpos(sLine, "<"), strpos(sLine, ">"));
	return sValue;
}

//------------------------------------------------------------------------------------------------------------------------

// unImportValidation_createDp
/**
Purpose: create datapoint

Parameters:
		@param sDpName, string, input, datapoint name
		@param sDpType, string , input, datapoint type
		@param exceptionInfo, dyn_string, output, for errors
		
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unImportValidation_createDp(string sDpName, string sDpType, dyn_string &exceptionInfo)
{
	string deviceName=sDpName;
	bool nameOk = false;
	int pos = strpos(sDpName,":");
	
	if(pos >=0)
		deviceName = substr(sDpName,strpos(sDpName,":")+1);
	if (dpExists(deviceName) == false)
	{
		dpCreate(deviceName, sDpType);
		if (dpExists(deviceName) == false)
		{
			fwException_raise(exceptionInfo,"ERROR","unImportValidation_createDp: " + deviceName + getCatStr("unGeneration","DPCREATEFAILED"),"");
		}
	}
}


//------------------------------------------------------------------------------------------------------------------------

// unImportValidation_importSilent
/**
Purpose: import in silent mode

Parameters:
		@param bKeepSystemName, bool , input, true=keep the system name for the plot DPE/false=replace the system name by #local:
		@param bImportInto, bool, input, true=import into sImportIntoNodeName/false=do not import into sImportIntoNodeName
		@param sImportIntoNodeName, string , input, the node name to import the data into
		@param dsTreeData, dyn_string , input, the data to import, WARNING \t are included
		@param exError, dyn_string, output, for errors
		@param exWarning, dyn_string, output, for warnings
		
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unImportValidation_importSilent(bool bKeepSystemName, bool bImportInto, string sImportIntoNodeName, string sFileName, 
																																						dyn_string &exError, dyn_string &exWarning)
{
	string sTag;
	dyn_string exInfoTemp, dsFileContent, dsData;
	int iIndex, len, iOldIndex, pos, i;
	bool bIgnore, bImport, bWarning, bOneWarning;
	bool bCheckTree=false;
	string sTreeName, sValue, sLine;
	bool bEmpty;
  bool bContinue = true;
  file fFile;
  int iLineNumber, iLineDevice;
	
  fFile = fopen(sFileName,"r"); //Create file for writing and reading; if a file with the given name already exists, its contents are lost.	
  if(fFile == 0)
  {
		fwException_raise(exError, "ERROR", "unImportValidation_importSilent: wrong file name "+sFileName, "");
		return;
  }						
  fgets(sLine,IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  strreplace(sLine, "\n", "");
  strreplace(sLine, "\r", "");
  strreplace(sLine, "\t", "");
  strreplace(sLine, "&amp;", "@");
  sLine = strltrim(strrtrim(sLine, " "), " ");
//        DebugN(sLine);
  dynAppend(dsFileContent, sLine);
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  strreplace(sLine, "\n", "");
  strreplace(sLine, "\r", "");
  strreplace(sLine, "\t", "");
  strreplace(sLine, "&amp;", "@");
  sLine = strltrim(strrtrim(sLine, " "), " ");
//        DebugN(sLine);
  dynAppend(dsFileContent, sLine);
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  strreplace(sLine, "\n", "");
  strreplace(sLine, "\r", "");
  strreplace(sLine, "\t", "");
  strreplace(sLine, "&amp;", "@");
  sLine = strltrim(strrtrim(sLine, " "), " ");
//        DebugN(sLine);
  dynAppend(dsFileContent, sLine);
	iIndex = 1;
	
	if(bImportInto && (sImportIntoNodeName == "")) {
		fwException_raise(exError, "ERROR", "unImportValidation_importSilent: empty node", "");
		return;
	}
	
	dsData = makeDynString();
	bImport = false;
	bWarning = false;
//	unImportValidation_getTreeData(dsFileContent, dsData, iIndex);
	unImportValidation_checkTreeData(dsFileContent, sTreeName, bKeepSystemName, exInfoTemp);
	if(sTreeName != "")
		bCheckTree = true;
	if(dynlen(exInfoTemp) >0) {
		if(dynContains(exInfoTemp, "ERROR") >0) 
			dynAppend(exError, exInfoTemp);
		else {
			bImport = true;
			bWarning = true;
			if(dynlen(dynPatternMatch("unImportValidation_checkTreeData: no tree definition*", exInfoTemp)) > 0) {
				bImport = true;
				bWarning = true;
				bEmpty = true;
			}
			else
				dynAppend(exWarning, exInfoTemp);
		}
	}
	else
		bImport = true;

	if(bImport) {
// importing tree
		exInfoTemp = makeDynString();
		unImportValidation_importTreeData(dsData, exInfoTemp);
		if(dynlen(exInfoTemp) >0) {
			if(dynContains(exInfoTemp, "ERROR") >0)
				dynAppend(exError, exInfoTemp);
		}
		else {
			if(bWarning) {
				bOneWarning = true;
			}
		}
	}

	if(bImport) {
		if(bImportInto) {
			if(sImportIntoNodeName != "") {
// replace the first instance of parentname by the selected parent.
				sValue = unImportValidation_getTagValue(dsFileContent, "<parentname>");
//DebugN(sValue, sImportIntoNodeName);
				pos = dynContains(dsFileContent, "<parentname>"+sValue+"</parentname>");
				if(pos>0) {
					strreplace(sImportIntoNodeName, "&", "@");
					dsFileContent[pos] = "<parentname>"+sImportIntoNodeName+"</parentname>";
				}
			}
		}
	}
//DebugN(dsData, iIndex, dsFileContent[iIndex]);
	while(bContinue) {
		dsData = makeDynString();
		exInfoTemp = makeDynString();
		sTag = "";
		iOldIndex = iIndex;
		bImport = false;
		bWarning = false;
		unImportValidation_getNextTagSectionDataFromFile(iLineNumber, iLineDevice, bContinue, fFile, dsFileContent, dsData, iIndex, sTag);
		switch(sTag) {
			case "":
				// do nothing
				break;
			case "<otherdevice>":
				// case of other devices, restart from iOldIndex+1: skip the <otherdevice> line and get the next tag section data
				iIndex = iOldIndex+1;
				break;
			default:
				unImportValidation_checkTagSectionData(bCheckTree, sTreeName, unImportValidation_getTagName(sTag), dsData, bKeepSystemName, exInfoTemp, iLineDevice, sFileName, false);
				if(bCheckTree) {
					bCheckTree = false;
					sTreeName = "";
				}
				if(dynlen(exInfoTemp) >0) {
					if(dynContains(exInfoTemp, "ERROR") >0)
						dynAppend(exError, exInfoTemp);
					else {
						bImport = true;
						bWarning = true;
						dynAppend(exWarning, exInfoTemp);
					}
//DebugN(sTag, iIndex, exInfoTemp, sText);
				}
				else
					bImport = true;

				if(bImport) {
					exInfoTemp = makeDynString();
					unImportValidation_importTagSectionData(unImportValidation_getTagName(sTag), dsData, exInfoTemp);
					if(dynlen(exInfoTemp) >0) {
						if(dynContains(exInfoTemp, "ERROR") >0)
							dynAppend(exError, exInfoTemp);
						else
							dynAppend(exWarning, exInfoTemp);
					}
					else {
						if(bWarning) {
							dynAppend(exWarning, exInfoTemp);
							bOneWarning = true;
						}
						else
							dynAppend(exWarning, exInfoTemp);
					}
				}
				break;
		}
	}
  fclose(fFile);
}

//------------------------------------------------------------------------------------------------------------------------

// unImportValidation_checkSilent
/**
Purpose: check in silent mode

Parameters:
		@param bKeepSystemName, bool , input, true=keep the system name for the plot DPE/false=replace the system name by #local:
		@param bImportInto, bool, input, true=import into sImportIntoNodeName/false=do not import into sImportIntoNodeName
		@param sImportIntoNodeName, string , input, the node name to import the data into
		@param dsTreeData, dyn_string , input, the data to import, WARNING \t are included
		@param exError, dyn_string, output, for errors
		@param exWarning, dyn_string, output, for warnings
		
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unImportValidation_checkSilent(bool bKeepSystemName, bool bImportInto, string sImportIntoNodeName, string sFileName, 
																																						dyn_string &exError, dyn_string &exWarning, bool bKeepInMemory=false)
{
	dyn_string exInfoTemp, dsFileContent, dsData;
	int iIndex, len, iOldIndex, pos, i;
	bool bIgnore, bImport, bWarning, bOneWarning;
	bool bCheckTree=false;
	string sTreeName, sValue, sTag, sLine;
	bool bEmpty;
  bool bContinue = true;
  file fFile;
  int iLineNumber, iLineDevice;
	
  fFile = fopen(sFileName,"r"); //Create file for writing and reading; if a file with the given name already exists, its contents are lost.							
  if(fFile == 0)
  {
		fwException_raise(exError, "ERROR", "unImportValidation_checkSilent: wrong file name "+sFileName, "");
		return;
  }						
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  strreplace(sLine, "\n", "");
  strreplace(sLine, "\r", "");
  strreplace(sLine, "\t", "");
  strreplace(sLine, "&amp;", "@");
  sLine = strltrim(strrtrim(sLine, " "), " ");
//        DebugN(sLine);
  dynAppend(dsFileContent, sLine);
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  strreplace(sLine, "\n", "");
  strreplace(sLine, "\r", "");
  strreplace(sLine, "\t", "");
  strreplace(sLine, "&amp;", "@");
  sLine = strltrim(strrtrim(sLine, " "), " ");
//        DebugN(sLine);
  dynAppend(dsFileContent, sLine);
  fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
  strreplace(sLine, "\n", "");
  strreplace(sLine, "\r", "");
  strreplace(sLine, "\t", "");
  strreplace(sLine, "&amp;", "@");
  sLine = strltrim(strrtrim(sLine, " "), " ");
//        DebugN(sLine);
  dynAppend(dsFileContent, sLine);
	iIndex = 1;
	
	if(bImportInto && (sImportIntoNodeName == "")) {
		fwException_raise(exError, "ERROR", "unImportValidation_checkSilent: empty node", "");
		return;
	}

	dsData = makeDynString();
//	unImportValidation_getTreeData(dsFileContent, dsData, iIndex);
	unImportValidation_checkTreeData(dsFileContent, sTreeName, bKeepSystemName, exInfoTemp);
	if(sTreeName != "")
		bCheckTree = true;
	if(dynlen(exInfoTemp) >0) {
		if(dynContains(exInfoTemp, "ERROR") >0) 
			dynAppend(exError, exInfoTemp);
		else {
			if(dynlen(dynPatternMatch("unImportValidation_checkTreeData: no tree definition*", exInfoTemp)) > 0) {
				bEmpty = true;
			}
			else
				dynAppend(exWarning, exInfoTemp);
		}
	}
//DebugN(dsData, iIndex, dsFileContent[iIndex]);
	if(bImportInto && bEmpty) {
		if(sImportIntoNodeName != "") {
// replace the first instance of parentname by the selected parent.
			sValue = unImportValidation_getTagValue(dsFileContent, "<parentname>");
//DebugN(sValue, sImportIntoNodeName);
			pos = dynContains(dsFileContent, "<parentname>"+sValue+"</parentname>");
			if(pos>0) {
				strreplace(sImportIntoNodeName, "&", "@");
				dsFileContent[pos] = "<parentname>"+sImportIntoNodeName+"</parentname>";
			}
		}
	}

	while(bContinue) {
		dsData = makeDynString();
		exInfoTemp = makeDynString();
		sTag = "";
		iOldIndex = iIndex;
		unImportValidation_getNextTagSectionDataFromFile(iLineNumber, iLineDevice, bContinue, fFile, dsFileContent, dsData, iIndex, sTag);
		switch(sTag) {
			case "":
				// do nothing
				break;
			case "<otherdevice>":
				// case of other devices, restart from iOldIndex+1: skip the <otherdevice> line and get the next tag section data
				iIndex = iOldIndex+1;
				break;
			default:
// only the first time check if the treename is the same as the first node name.
//DebugN(bCheckTree, sTreeName);
				unImportValidation_checkTagSectionData(bCheckTree, sTreeName, unImportValidation_getTagName(sTag), dsData, bKeepSystemName, exInfoTemp, iLineDevice, sFileName, bKeepInMemory);
				if(bCheckTree) {
					bCheckTree = false;
					sTreeName = "";
				}
				if(dynlen(exInfoTemp) >0) {
					if(dynContains(exInfoTemp, "ERROR") >0)
						dynAppend(exError, exInfoTemp);
					else
						dynAppend(exWarning, exInfoTemp);
//DebugN(sTag, iIndex, exInfoTemp, sText);
				}
				break;
		}
	}
  fclose(fFile);
}

//------------------------------------------------------------------------------------------------------------------------

unImportValidation_getNextTagSectionDataFromFile(int &iLineNumber, int &iLineDevice, bool &bContinue, file fFile, dyn_string &dsFileContent, dyn_string &dsData, int &iStartIndex, string &sTag)
{
  /*
				fgets(sLine, MAX_CHAR, fFile);
/*
      			while(feof(fFile) == 0) {
				fgets(sLine, MAX_CHAR, fFile);
				strreplace(sLine, "\n", "");
				strreplace(sLine, "\r", "");
				strreplace(sLine, "\t", "");
        strreplace(sLine, "&amp;", "@");
        sLine = strltrim(strrtrim(sLine, " "), " ");
//        DebugN(sLine);
				dynAppend(dsFileContent, sLine);
    */
	bool bFound=false;
	dyn_string dsSplit;
	int len = dynlen(dsFileContent);
	string sTempStartTag, sTempEndTag;
  int iIndex=1;
  string sLine;
	
	while(!bFound && (feof(fFile) == 0)) {
//DebugN(dsFileContent[iIndex], strpos(dsFileContent[iIndex], ">"), strlen(dsFileContent[iIndex]), strpos(dsFileContent[iIndex], "<"), strpos(dsFileContent[iIndex], "/"));
//return;
				fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;iLineDevice = iLineNumber;
				strreplace(sLine, "\n", "");
				strreplace(sLine, "\r", "");
				strreplace(sLine, "\t", "");
        strreplace(sLine, "&amp;", "@");
        sLine = strltrim(strrtrim(sLine, " "), " ");
        if(sLine != "<otherdevice>")
        {
//        DebugN("!!!!!!!!!!!!!!", sLine);
//				dynAppend(dsFileContent, sLine);
        
		if(((strpos(sLine, ">")+1) == strlen(sLine)) &&
				(strpos(sLine, "<") == 0) &&
				(strpos(sLine, "/") < 0)
			) {
// this is a tag
//DebugN("tag", iIndex, dsFileContent[iIndex]);
				bFound = true;
				sTempStartTag = sLine;
		}
		else
			iIndex++;
  }
//        else
//          DebugN("<otherdevice>");
	}
	if(bFound) {
		unImportValidation_getTagSectionDataFromFile(iLineNumber, fFile, dsFileContent, dsData, iIndex, sTempStartTag, "</"+unImportValidation_getTagName(sTempStartTag)+">");
		sTag = sTempStartTag;
	}
  iStartIndex = 1;
  if(feof(fFile) != 0)
    bContinue = false;
}

unImportValidation_getTagSectionDataFromFile(int &iLineNumber, file fFile, dyn_string &dsFileContent, dyn_string &dsData, int &iStartIndex, string sStartTag, string sEndTag)
{
	int i, j, len=dynlen(dsFileContent);
	bool bNotFound=true;
  string sLine;

//  DebugN(len, sStartTag, sEndTag);
// search the startTag tag from iIndex
  while((feof(fFile) == 0) && bNotFound)
  {
//				fgets(sLine, MAX_CHAR, fFile);
//				strreplace(sLine, "\n", "");
//				strreplace(sLine, "\r", "");
//				strreplace(sLine, "\t", "");
//        strreplace(sLine, "&amp;", "@");
//        sLine = strltrim(strrtrim(sLine, " "), " ");
//     DebugN("!! sec", sLine);
//        if(sLine == sStartTag)
//        {
          while((feof(fFile) == 0) && bNotFound)
          {
    				fgets(sLine, IMPORTTREE_TREE_MAX_CHAR, fFile);iLineNumber++;
    				strreplace(sLine, "\n", "");
    				strreplace(sLine, "\r", "");
    				strreplace(sLine, "\t", "");
            strreplace(sLine, "&amp;", "@");
            sLine = strltrim(strrtrim(sLine, " "), " ");
    				if(sLine != sEndTag) {
    					dynAppend(dsData, sLine);
            }
    				else {
    					bNotFound=false;
    				}
          }
        }
//      }
  iStartIndex = 1;
//DebugN("bnotfound", bNotFound, iIndex);
}
dyn_string unImportValidation_getAllPanel()
{
  dyn_string dsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    g_dsPanel_Copy = g_dsPanel;
    dsReturn = g_dsPanel_Copy;
    dynSortAsc(dsReturn);
    dynUnique(dsReturn);
    g_ddsPanelData_Copy = g_ddsPanelData;
  }
  return dsReturn;
}
dyn_string unImportValidation_getAllPage()
{
  dyn_string dsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    g_dsPage_Copy = g_dsPage;
    dsReturn = g_dsPage_Copy;
    dynSortAsc(dsReturn);
    dynUnique(dsReturn);
    g_ddsPageData_Copy = g_ddsPageData;
  }
  return dsReturn;
}
dyn_string unImportValidation_getAllPlot()
{
  dyn_string dsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    g_dsPlot_Copy = g_dsPlot;
    dsReturn = g_dsPlot_Copy;
    dynSortAsc(dsReturn);
    dynUnique(dsReturn);
    g_ddsPlotData_Copy = g_ddsPlotData;
  }
  return dsReturn;
}
dyn_string unImportValidation_getAllTreeNode()
{
  dyn_string dsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    g_dsTreeNode_Copy = g_dsTreeNode;
    dsReturn = g_dsTreeNode_Copy;
    dynSortAsc(dsReturn);
    dynUnique(dsReturn);
    g_ddsTreeNodeData_Copy = g_ddsTreeNodeData;
  }
  return dsReturn;
}
dyn_string unImportValidation_getAllNavigation()
{
  dyn_string dsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    g_dsNavigation_Copy = g_dsNavigation;
    dsReturn = g_dsNavigation_Copy;
    dynSortAsc(dsReturn);
    dynUnique(dsReturn);
    g_ddsNavigationData_Copy = g_ddsNavigationData;
  }
  return dsReturn;
}
dyn_dyn_string _unImportValidation_getData(dyn_string &dsData, dyn_dyn_string &ddsData, string sKey, dyn_string &dsLineNumber, dyn_string &dsDeviceType, dyn_string &dsFileName)
{
  dyn_string dsReturn;
  dyn_dyn_string ddsReturn;
  int pos;
  string sDeviceType, sLineNumber, sFileName;
  
  dynClear(dsDeviceType);
  dynClear(dsLineNumber);
  dynClear(dsFileName);
  pos = dynContains(dsData, sKey);
  while(pos>0)
  {
    dsReturn = ddsData[pos];
    while(dynlen(dsReturn) <= IMPORTTREE_TREE_LINE_NUMBER)
      dynAppend(dsReturn, "-ERROR-NOT-SET-");
    sDeviceType = dsReturn[IMPORTTREE_TREE_NODE_TYPE];
    sLineNumber = dsReturn[IMPORTTREE_TREE_LINE_NUMBER];
    sFileName = dsReturn[IMPORTTREE_TREE_FILENAME];
    dynRemove(dsReturn, 1);
    dynRemove(dsReturn, 1);
    dynRemove(dsReturn, 1);
    dynAppend(dsDeviceType, sDeviceType);
    dynAppend(dsLineNumber, sLineNumber);
    dynAppend(dsFileName, sFileName);
    dynAppend(ddsReturn, dsReturn);
    dynRemove(dsData, pos);
    dynRemove(ddsData, pos);
    pos = dynContains(dsData, sKey);
  }
  return ddsReturn;
}
dyn_dyn_string unImportValidation_getTreeNodeData(string sKey, dyn_string &dsLineNumber, dyn_string &dsDeviceType, dyn_string &dsFileName)
{
  dyn_dyn_string ddsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    ddsReturn = _unImportValidation_getData(g_dsTreeNode_Copy, g_ddsTreeNodeData_Copy, sKey, dsLineNumber, dsDeviceType, dsFileName);
  }
  return ddsReturn;
}
dyn_dyn_string unImportValidation_getPanelData(string sKey, dyn_string &dsLineNumber, dyn_string &dsDeviceType, dyn_string &dsFileName)
{
  dyn_dyn_string ddsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    ddsReturn = _unImportValidation_getData(g_dsPanel_Copy, g_ddsPanelData_Copy, sKey, dsLineNumber, dsDeviceType, dsFileName);
  }
  return ddsReturn;
}
dyn_dyn_string unImportValidation_getPlotData(string sKey, dyn_string &dsLineNumber, dyn_string &dsDeviceType, dyn_string &dsFileName)
{
  dyn_dyn_string ddsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    ddsReturn = _unImportValidation_getData(g_dsPlot_Copy, g_ddsPlotData_Copy, sKey, dsLineNumber, dsDeviceType, dsFileName);
  }
  return ddsReturn;
}
dyn_dyn_string unImportValidation_getPageData(string sKey, dyn_string &dsLineNumber, dyn_string &dsDeviceType, dyn_string &dsFileName)
{
  dyn_dyn_string ddsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    ddsReturn = _unImportValidation_getData(g_dsPage_Copy, g_ddsPageData_Copy, sKey, dsLineNumber, dsDeviceType, dsFileName);
  }
  return ddsReturn;
}
dyn_dyn_string unImportValidation_getNavigationData(string sKey, dyn_string &dsLineNumber, dyn_string &dsDeviceType, dyn_string &dsFileName)
{
  dyn_dyn_string ddsReturn;
  synchronized(g_ddsTreeNodeData)
  {
    ddsReturn = _unImportValidation_getData(g_dsNavigation_Copy, g_ddsNavigationData_Copy, sKey, dsLineNumber, dsDeviceType, dsFileName);
  }
  return ddsReturn;
}

unImportValidation_clearMemoryData()
{
  dynClear(g_dsTreeNode);
  dynClear(g_dsPanel);
  dynClear(g_dsPage);
  dynClear(g_dsPlot);
  dynClear(g_dsNavigation);

  dynClear(g_ddsTreeNodeData);
  dynClear(g_ddsPanelData);
  dynClear(g_ddsPageData);
  dynClear(g_ddsPlotData);
  dynClear(g_ddsNavigationData);
}




//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function collects the Aliases from the Database Instances files.
 *@param string sInstancesImportFile-[IN] Database Instances file name(full path)/
 *@return dyn_dyn_string collectedDB- Contains all the aliases from the Database Instances file 
 *        collectedDB[1]:Device Type and collectedDB[2]: Aliases.
*/

dyn_dyn_string unImportValidation_dbCollector(string sInstancesImportFile)
{ dyn_string DataString,tempAlias,collectedDBAliases,collectAlias,type;
  dyn_dyn_string collectedDB;
  string sLine;
  file f;
  int i,numberOfLine=1;
  
  i=access(sInstancesImportFile,F_OK); // Check if the file exists
 
  if (i!=0) // File does not exist
 
{
    fprintf(fReport, "WARNING %s: The file does not exist\n", sInstancesImportFile);
   
}
  else
  {
    f=fopen(sInstancesImportFile,"r");// open the file
    while (feof(f)==0)
    {
    fgets(sLine,10000, f);//get each line's content
    strreplace(sLine, "\r", "");		//replace carriage with empty
    strreplace(sLine, "\n", "");		// replace line feed with empty
    if((strrtrim(strltrim(sLine)) != "") && (substr(sLine, 0, 1) != "#"))//check if the line contains # or empty
    {
    DataString= strsplit(sLine," ;");//split the content of each line
    if(patternMatch("*,*",DataString[3]))//check if the alias contains a Device link
    {
     tempAlias=strsplit(DataString[3]," ,");//if yes split and take the alias only
     collectAlias=tempAlias[1];
     dynAppend(type,DataString[1]);
    }else
     {     
     collectAlias=DataString[3];//take the alias
     dynAppend(type,DataString[1]);
     }
     dynAppend(collectedDBAliases,collectAlias);
     }  
    numberOfLine++;
    }
   fclose(f);
  }
  collectedDB[1]=type;
  collectedDB[2]=collectedDBAliases;
  return  collectedDB;//collectedDB[1]:Device Type and collectedDB[2]: Aliases
}

//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function collects the Dpe types,defined by UNICOS, that have the ability to be trended.
 *@return dyn_string collectedTrenedDpeTypes which contains all Dpe types,defined by UNICOS, that have the ability 
*         to be trended.
*/
dyn_string unImportValidation_collectDpeTrendedTypes()
{
 dyn_string UnicosObjects,collectedTrenedDpeTypes,tempDpeTrendedType,elements;
 
  unGenericDpFunctions_getUnicosObjects(UnicosObjects);//get all the Unicos object types
  for (int i=1; i<=dynlen(UnicosObjects); i++)
  { dpGet(UnicosObjects[i]+"_unicosInfo.trendingDpSelector.DpeList",elements);//get the allowed trended DPEs for each Unicos object type
    for(int j = 1; j<=dynlen(elements); j++)
    {if (elements[j]!="") //if there are allowed trended DPEs
   { tempDpeTrendedType= strsplit(elements[j]," ;");//split the content and take the first ex.PosSt;PosSt
   dynAppend(collectedTrenedDpeTypes,UnicosObjects[i]+"."+tempDpeTrendedType[1]);//collect the trended DPE types
   }
  }
  }
  return collectedTrenedDpeTypes;//example of the output format deviceType.DPE ex."CPC_AnalogAlarm.PosSt"
}

//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function collects .xml and .pnl files in the "panels" and panels/vision" folders
 *@param dyn_string &xmlFiles-[OUT] List of .xml files(full path)
 *@param dyn_string &pnlFiles-[OUT] List of .pnl files(full path)
*/

void unImportValidation_collectPanels(dyn_string &xmlFiles,dyn_string &pnlFiles)     
{
  unImportValidation_specifiedFolderCollectPanels(xmlFiles, pnlFiles);
}

//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function returns all the folders of a folder (recursive function)
 *@param string sFolder-[IN] the folder
 *@param dyn_string &dsFolders-[OUT] List of all the folders in sFolder
*/
unImportValidation_getAllFolders(string sFolder, dyn_string &dsFolders)
{
  int i, len;
  dyn_string dsTemp;
  
  dsTemp = getFileNames(sFolder, "*", FILTER_DIRS);
  i = dynContains(dsTemp, ".");
  if(i>0)
    dynRemove(dsTemp, i);
  i = dynContains(dsTemp, "..");
  if(i>0)
    dynRemove(dsTemp, i);
  len = dynlen(dsTemp);
//  DebugN("start looking into "+sFolder,len, dsTemp);
  for(i=1;i<=len;i++)
  {
    unImportValidation_getAllFolders(sFolder+"/"+dsTemp[i], dsFolders);
  }
  dynAppend(dsFolders, sFolder);
//  DebugN("done folder", sFolder, dsFolders);
}

//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function collects .xml and .pnl files from the user specified folder(s)
 *@param dyn_string &specifiedXmlFiles-[OUT] List of .xml files(full path)
 *@param dyn_string &specifiedPnlFiles-[OUT] List of .pnl files(full path)
*/
void unImportValidation_specifiedFolderCollectPanels(dyn_string &specifiedXmlFiles, dyn_string &specifiedPnlFiles)  //collect the .xml and .pnl panel files from the user specified folders
{
   int i,j,m, len; 
   string  panelsDirectory,selectedFolderDirectory;
   dyn_int selectedLines;
   dyn_string s,tempXmlfiles,tempPnlfiles,selectedDpes;
   dyn_string dsFolders;
   
   getValue("panelFolderTbl","getSelectedLines",selectedLines);// get the specified folders
   panelsDirectory=rootFolder.text;//get the "panels" directory
   // get all folders and sub-folders
   for(int i = 1; i<=dynlen(selectedLines); i++)//for each specified folder
   {
     selectedFolderDirectory=panelsDirectory+panelFolderTbl.getLineN(selectedLines[i]);
     unImportValidation_getAllFolders(selectedFolderDirectory, dsFolders);
   }
//   DebugN(selectedFolderDirectory, dsFolders);
   len = dynlen(dsFolders);
   
   // get all panels and xml panels
   for(int i = 1; i<=len; i++)//for each specified folder
  {
//   DebugN(selectedFolderDirectory);
   
   tempXmlfiles=getFileNames(dsFolders[i], "*.xml");//collect the .xml files
    for(j=1; j<=dynlen(tempXmlfiles); j++) 
   { dynAppend(specifiedXmlFiles,dsFolders[i]+"/"+tempXmlfiles[j]);
   }
   
   tempPnlfiles=getFileNames(dsFolders[i], "*.pnl");//collect the .pnl files
   for(m=1; m<=dynlen(tempPnlfiles); m++) 
   { dynAppend(specifiedPnlFiles,dsFolders[i]+"/"+ tempPnlfiles[m]);
   }
  }
//  DebugN(specifiedXmlFiles);
//  DebugN(specifiedPnlFiles);
 }

//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function collects all the references information existing on a xml panel
 *@param string sfileName-[IN] panels' file directory
 *@return dyn_dyn_string collectedXmlReferences-[OUT] List of references
*/

dyn_dyn_string unImportValidation_xmlPanelParser(string sfileName)
{ string errMsg; 
  int fileId,nestedId,doubleNestedId,Result,errLine, errColumn; 
  dyn_int elementsId;
  dyn_string nodeNames,nestedNodeNames,doubleNestedNodeNames,nodeValues,nestedNodeValues,doubleNestedNodeValues,exceptionInfo;
  dyn_string collectedAtrributes,collectedNodeValues;
  dyn_anytype attributes,nestedAttributes,doubleNestedAttributes;
  dyn_dyn_string collectedXmlReferences;
  
  fileId=xmlDocumentFromFile(sfileName,errMsg,errLine,errColumn); //open the xml file
  if (fileId<0)
   {
    fprintf(fReport, "WARNING xml error ID: %d\nErrors: %s, %d, %d\n", fileId, errMsg, errLine, errColumn);
//    DebugN("The ID:", fileId);
//    DebugN("Errors:", errMsg, errLine, errColumn); 
   }else{
    //DebugN("The ID:", fileId); 
    elementsId=fwXml_elementsByTagName(fileId,-1,"properties",exceptionInfo);//collect the elements
    // DebugN("elementsId:",elementsId);
    for (int i=1;i<=dynlen(elementsId);i++)
    {fwXml_childNodesContent(fileId,elementsId[i],nodeNames,attributes,nodeValues,exceptionInfo);   //collect the child nodes  
     for (int j=1;j<=dynlen(nodeNames);j++)
      { 
      if ( mappingHasKey ( attributes[j] , fwXml_CHILDNODESTYPE ) )//check if there are nested children
       { //DebugN("It's nested");
        nestedId = attributes[j][fwXml_CHILDSUBTREEID];
        fwXml_childNodesContent(fileId,nestedId,nestedNodeNames,nestedAttributes,nestedNodeValues,exceptionInfo);//collect the nested child nodes 
        for (int k=1;k<=dynlen(nestedNodeNames);k++)
        {
          if ( mappingHasKey ( nestedAttributes[k] , fwXml_CHILDNODESTYPE ) )//check if there are double nested children
          {doubleNestedId=nestedAttributes[k][fwXml_CHILDSUBTREEID];
           fwXml_childNodesContent(fileId,doubleNestedId,doubleNestedNodeNames,doubleNestedAttributes,doubleNestedNodeValues,exceptionInfo); //collect the double nested child nodes 
           for (int m=1;m<=dynlen(doubleNestedNodeNames);m++) 
           {if (patternMatch("name:FileName",doubleNestedAttributes[m]))
            {//DebugN(doubleNestedNodeValues[m]);
             dynAppend(collectedAtrributes,doubleNestedAttributes[m]);
             dynAppend(collectedNodeValues,doubleNestedNodeValues[m]);
            }else if(patternMatch("name:Value",doubleNestedAttributes[m]))
            {//DebugN(doubleNestedNodeValues[m]);
             dynAppend(collectedAtrributes,doubleNestedAttributes[m]);
             dynAppend(collectedNodeValues,doubleNestedNodeValues[m]);
            }
           }
          }
        }
       }else
      {if (patternMatch("name:FileName",attributes[j]))
       {//DebugN(nodeValues[j]);
         dynAppend(collectedAtrributes,attributes[j]);
         dynAppend(collectedNodeValues,nodeValues[j]);
       }else if(patternMatch("name:Value",attributes[j]))
       {//DebugN(nodeValues[j]);
         dynAppend(collectedAtrributes,attributes[j]);
         dynAppend(collectedNodeValues,nodeValues[j]);
      }
      }
    }
    }
   }
   collectedXmlReferences[1]=collectedAtrributes;
   collectedXmlReferences[2]=collectedNodeValues;
   return collectedXmlReferences;
 }

//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function collects all the references information existing on a pnl panel
 *@param string sPnlFileName-[IN] panels' file directory
 *@return dyn_dyn_string collectedPnlReferences-[OUT] List of references
*/
dyn_dyn_string unImportValidation_pnlPanelParser(string sPnlFileName)
{ string sLine;
  dyn_string lineReference,reference,tempReference;
  dyn_dyn_string collectedPnlReferences;
  file f;
  int i,numberOfLine=1;
  
  i=access(sPnlFileName,F_OK); // Check if the file exists
 
  if (i!=0) // File does not exist
  {
    fprintf(fReport, "WARNING %s: The file does not exist\n", sPnlFileName);
  }else
  {f=fopen(sPnlFileName,"r");// open the file
   while (feof(f)==0)
   {fgets(sLine,10000, f);//get each line's content
    strreplace(sLine, "\r", "");		//replace carriage with empty
    strreplace(sLine, "\n", "");		// replace line feed with empty
    if((strrtrim(strltrim(sLine)) != "") && (substr(sLine, 0, 1) != "#"))//check if the line contains # or empty
     {if(substr(sLine, 1, 1) == "$")//check if the line contains a $parameter
      {tempReference=strsplit(sLine,"\"");
       for(int i = 1; i<=dynlen(tempReference); i++)
        { if((tempReference[i]!="")&& !patternMatch("*$*",tempReference[i]))
         {dynAppend(reference,tempReference[i]);//collect the $parameter and the line number
          dynAppend(lineReference,numberOfLine);
         }
       }
      }else
       {if(patternMatch("*/*.*",sLine))//check if the line contains a reference to an object
        {tempReference=strsplit(sLine,"\"");
          for(int i = 1; i<=dynlen(tempReference); i++)
           {if((tempReference[i]!="")&& patternMatch("*/*.*",tempReference[i]))
            {dynAppend(reference,tempReference[i]);//collect the object reference and the line number
             dynAppend(lineReference,numberOfLine);
              
            }
           }
        }
       }
     } 
    numberOfLine++;
    }
   fclose(f);
  }
  collectedPnlReferences[1]=lineReference;
  collectedPnlReferences[2]=reference;
  
  return collectedPnlReferences;
                   
}

//--------------------------------------------------------------------------------------------------------------------------
/**
 *The function writes in the output report file 
 *@param dyn_string &specifiedXmlFiles-[OUT] List of .xml files(full path)
 *@param dyn_string &specifiedPnlFiles-[OUT] List of .pnl files(full path)
*/
void unImportValidation_report(int errorNumber,string sItem,string sfile,dyn_string sline,int smode)
{string errorMessage;
 if(smode==1)//mode:1 is for panel report
  {if(errorNumber==1)
  {errorMessage="[ERROR Type:1] The Alias "+sItem+" in the file "+sfile+" ,does not exist in the instance importation file(s) \n"; 
  fprintf(fReport, "%s",errorMessage);  
  }else if(errorNumber==3)
  {errorMessage="[ERROR Type:3] The Panel Reference "+sItem+" in the file "+sfile+" ,does not exist in the panels' folder \n";
  fprintf(fReport, "%s",errorMessage);    
  }else if(errorNumber==5)
  {errorMessage="[ERROR Type:5] The Alias "+sItem+" in the file "+sfile+" ,does not exist in the instance importation file(s) and CONTAINS a System name \n";
  fprintf(fReport, "%s",errorMessage);    
  }else if(errorNumber==6)
  {errorMessage="[WARNING] The Alias "+sItem+" in the file "+sfile+" ,CONTAINS a System name \n";
  fprintf(fReport, "%s",errorMessage);    
  } 
  }else if(smode==2)//mode:2 is for tree report
 {if(errorNumber==1)
  {errorMessage="[ERROR Type:1] The Alias "+sItem+" in the file "+sfile+", line "+sline+", does not exist in the instance importation file(s) \n"; 
  fprintf(fReport, "%s",errorMessage);  
  }else if(errorNumber==2)
  {errorMessage="[ERROR Type:2] The Data Point Element "+sItem+" in the file "+sfile+", line "+sline+", can not be trended according to UNICOS standards \n"; 
  fprintf(fReport, "%s",errorMessage);    
  }else if(errorNumber==3)
  {errorMessage="[ERROR Type:3] The Panel Reference "+sItem+" in the file "+sfile+", line "+sline+", does not exist in the panels' folder \n";
  fprintf(fReport, "%s",errorMessage);    
  }else if(errorNumber==4)
  {errorMessage="[ERROR Type:4] The Plot/Page Reference "+sItem+" in the file "+sfile+", line "+sline+", does not exist in the trend tree file \n";
  fprintf(fReport, "%s",errorMessage);    
  }else if(errorNumber==5)
  {errorMessage="[ERROR Type:5] The Alias "+sItem+" in the file "+sfile+" ,does not exist in the instance importation file(s) and CONTAINS a System name \n";
  fprintf(fReport, "%s",errorMessage);    
  }else if(errorNumber==6)
  {errorMessage="[WARNING] The Alias "+sItem+" in the file "+sfile+" ,CONTAINS a System name \n";
  fprintf(fReport, "%s",errorMessage);    
  }
 }
}

//@}


