/**
@file 
@date        30/09/2016
@author      Riku-Pekka Silvola
@copyright   CERN (BE-ICS-SDS) 

@brief Library contains utility functions used by the unLHCLaser component.

Usage: Public

PVSS manager usage: CTRL, UI

Constraints:
- PVSS version: 3.14
- operating system: Linux.
*/

/**
* Concatenates a list of elements. Works on any datatypes that can be casted to a string.
*
* @param triplet   input, mapping of triplet strings to check (format: FF:FM:FC)
*
* @return          output, concatenated string
*/
string unLaser_dynConcatenate(dyn_mixed elements, string join_string = "", string prefix = "") {
  string conc_string;
  unsigned nof_elements = dynlen(elements);
  if (nof_elements == 1) {
    return (string)elements[1];
  }
  else if (nof_elements > 0) {
    conc_string = prefix + (string)elements[1];
    for (unsigned element_idx = 2; element_idx < nof_elements; element_idx += 1) {
      conc_string += join_string + prefix + (string)elements[element_idx];
    }
  }
  return conc_string;
}
